import pyximport; pyximport.install()

import importlib
import logging
import os
from os.path import join, dirname
import shlex

from cdecimal import Decimal
from cement.core import foundation, controller, handler
from cement.ext.ext_argparse import ArgparseArgumentHandler
from dotenv import load_dotenv

from gryphon.execution.lib import config_helper
from gryphon.lib import environment
from gryphon.lib.logger import get_logger


environment.load_environment_variables()


def turn_on_debug(loggers):
    for logger_name in loggers:
        logger = get_logger(logger_name)
        logger.setLevel(logging.DEBUG)


class ModifiedArgparseArgumentHandler(ArgparseArgumentHandler):
    class Meta:
        label = 'my_argparse'
        ignore_unknown_arguments = True


class GryphonFuryBaseController(controller.CementBaseController):
    """
    The application base controller.
    """

    class Meta:
        label = 'base'

        config_defaults = {
            'strategy': 'Naive',
            'exchange': None,
            'our_orders': False,
            'execute': False,
        }

        arguments = [
            (['strategy'], {
                'action': 'store',
                'help': 'name of strategy',
            }),
            (['-b', '--builtin'], {
                'action': 'store_true',
                'help': 'Whether the strategy name specified refers to a built-in strategy in the Gryphon Framework',
            }),
            (['-X', '--execute'], {
                'action': 'store_true',
                'help': 'execute real trades',
            }),
            (['--heartbeat'], {
                'action': 'store_true',
                'default': None,
                'help': 'heartbeat every tick for monitoring',
            }),
            (['--sentry'], {
                'action': 'store_true',
                'default': None,
                'help': 'log exceptions to a 3rd party service',
            }),
            (['--emerald'], {
                'action': 'store_true',
                'default': None,
                'help': 'look for cached market data in redis vs. polling in-process',
            }),
            (['--audit'], {
                'action': 'store_true',
                'default': None,
                'help': 'whether to run the strategy with audits or not',
            }),
            (['--tick_sleep'], {
                'action': 'store',
                'help': 'How many seconds to sleep between ticks',
            }),
            (['--more-logging'], {
                'action': 'store_true',
                'help': 'extra debug output',
            }),
            (['-c', '--config_file'], {
                'action': 'store',
                'help': 'relative path to the configuration file',
            }),
        ]

    def _parse_strategy_args(self, strat_args):
        """
        Strategy arguments currently come in the form --strat_args [k=v,k2=v2,...].
        This could use some more work for elegance, but this function gives us fairly
        smart parsing of this format, such that complex forms like the following still
        work.

        --strat_args 'exchange=kraken,fundamental_exchanges="kraken,kraken_usd"'
        """
        parsed_args = {}

        if strat_args is not None:
            lexer = shlex.shlex(strat_args, posix=True)
            lexer.whitespace_split = True
            lexer.whitespace = ','

            parsed_args = dict(pair.split('=', 1) for pair in lexer)

        return parsed_args

    @controller.expose()
    def strategy(self):
        if self.app.pargs.more_logging:
            loggers = [
                'requests.packages.urllib3',
                'gryphon.lib.exchange.base',
                'strategies.harness',
                'strategies.base',
            ]

            turn_on_debug(loggers)

        file_configuration = config_helper.get_conf_file_configuration(
            self.app.pargs.config_file,
            self.app.pargs.strategy,
        )

        command_line_configuration = config_helper.get_command_line_configuration(
            self.app.pargs,
            self.app.args.unknown_args, 
        )

        final_configuration = config_helper.combine_file_and_command_line_config(
            file_configuration,
            command_line_configuration,
        )

        from gryphon.execution.live_runner import live_run

        self.app.log.info(
            'Running strategy %s with configuration=%s' % (
            self.app.pargs.strategy,
            final_configuration,
        ))

        live_run(final_configuration)

    @controller.expose()
    def overwatch(self):
        from gryphon.execution.bots import overwatch
        overwatch.watch()

    @controller.expose()
    def shoebox(self):
        from gryphon.execution.bots import shoebox
        shoebox.run()

    @controller.expose()
    def bank(self):
        from gryphon.execution.bots import bank
        bank.run()

    @controller.expose()
    def buyback(self):
        from gryphon.execution.controllers.fee_buyback import buyback
        buyback()

    @controller.expose()
    def balance(self):
        from gryphon.execution.controllers.balance import balance

        exchange_key = self.app.config.get('controller.base', 'exchange')
        balance(exchange_key)

    @controller.expose()
    def winddown(self):
        from gryphon.execution.controllers.wind_down import wind_down

        exchange_key = self.app.config.get('controller.base', 'exchange')
        self.app.config.parse_file('%s.conf' % exchange_key)
        strategy_params = self.app.config.get_section_dict('live')
        execute = self.app.config.get('controller.base', 'execute')
        wind_down(exchange_key, strategy_params, execute)


class OrderBookController(controller.CementBaseController):
    """This controller's commands are 'stacked' onto the base controller."""

    class Meta:
        label = 'orderbook'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'OrderBook Controller'
        arguments = [
            (['exchange'], {
                'action': 'store',
                'help': 'name of exchange',
                'nargs': '?',
            }),
            (['--our-orders'], {'action': 'store_true', 'help': 'highlight our orders'}),
            (['--include-fees'], {'action': 'store_true', 'help': 'include fees in prices'}),
        ]

    @controller.expose(help='show individual or combined orderbooks')
    def default(self):
        from gryphon.execution.controllers.order_book import order_book

        exchange_key = self.app.pargs.exchange
        our_orders = self.app.pargs.our_orders
        include_fees = self.app.pargs.include_fees
        order_book(exchange_key, our_orders, include_fees)


class AuditController(controller.CementBaseController):
    """This controller commands are 'stacked' onto the base controller."""

    class Meta:
        label = 'audit'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'Audit Controller'
        arguments = [
            (['exchange'], {
                'action': 'store',
                'help': 'name of exchange',
                'nargs': '?',
            }),
            (['--more-logging'], {
                'action': 'store_true',
                'help': 'extra debug output',
            }),
        ]

    @controller.expose(help='compare exchange trades to our records')
    def default(self):
        from gryphon.execution.lib.auditing import audit

        exchange_key = self.app.pargs.exchange

        if self.app.pargs.more_logging:
            loggers = ['gryphon.lib.models.exchange', 'utils.audit']
            turn_on_debug(loggers)

        audit(exchange_key)


class ManualController(controller.CementBaseController):
    """This controller commands are 'stacked' onto the base controller."""

    class Meta:
        label = 'manual'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'Manual Controller'
        arguments = [
            (['exchange'], {
                'action': 'store',
                'help': 'name of exchange',
                'nargs': '?',
            }),
            (['order_id'], {'action': 'store', 'help': 'exchange order id to run accounting for'}),
            (['--actor'], {'action': 'store', 'help': 'actor name', 'default': 'Multi'}),
            (['--execute'], {'action': 'store_true', 'help': 'really save to the db'}),
        ]

    @controller.expose(help='run manual accounting on given order id')
    def default(self):
        from gryphon.execution.controllers.manual_accounting import manual_accounting

        exchange_key = self.app.pargs.exchange
        order_id = self.app.pargs.order_id
        actor = self.app.pargs.actor
        execute = self.app.pargs.execute
        manual_accounting(exchange_key, order_id, actor, execute)


class WithdrawController(controller.CementBaseController):
    """This controller commands are 'stacked' onto the base controller."""

    class Meta:
        label = 'withdraw'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'Withdraw Controller'
        arguments = [
            (['exchange'], {
                'action': 'store',
                'help': 'name of exchange',
                'nargs': '?',
            }),
            (['target_exchange'], {
                'action': 'store',
                'help': 'exchange where bitcoins were sent',
            }),
            (['amount'], {
                'action': 'store',
                'help': 'amount as a Money string eg "BTC 10"',
            }),
        ]

    @controller.expose(help='record a withdrawal')
    def default(self):
        from gryphon.execution.controllers.withdraw import withdraw

        exchange_key = self.app.pargs.exchange
        target_exchange_key = self.app.pargs.target_exchange
        amount = self.app.pargs.amount
        withdraw(exchange_key, target_exchange_key, amount)


class WithdrawFiatController(controller.CementBaseController):
    """This controller commands are 'stacked' onto the base controller."""

    class Meta:
        label = 'withdrawfiat'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'Withdraw Fiat Controller'
        arguments = [
            (['exchange'], {
                'action': 'store',
                'help': 'name of exchange',
                'nargs': '?',
            }),
            (['target_exchange'], {
                'action': 'store',
                'help': 'exchange where fiat was sent',
            }),
            (['amount'], {
                'action': 'store',
                'help': 'amount as a Money string eg "USD 100000"',
            }),
            (['external_transaction_id'], {
                'action': 'store',
                'help': 'eg. wire confirmation code',
            }),
            (['destination_amount'], {
                'action': 'store',
                'help': 'expected destination amount (if different than sent amount)',
                'nargs': '?',
            }),
        ]

    @controller.expose(help='record a fiat withdrawal')
    def default(self):
        from gryphon.execution.controllers.withdraw import withdraw_fiat

        exchange_key = self.app.pargs.exchange
        target_exchange_key = self.app.pargs.target_exchange
        amount = self.app.pargs.amount
        destination_amount = self.app.pargs.destination_amount
        external_transaction_id = self.app.pargs.external_transaction_id

        withdraw_fiat(
            exchange_key,
            target_exchange_key,
            amount,
            destination_amount,
            transaction_details={'external_transaction_id': external_transaction_id},
        )


class TransactionCompleteController(controller.CementBaseController):
    """This controller commands are 'stacked' onto the base controller."""

    class Meta:
        label = 'complete'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'Complete Transaction Controller'
        arguments = [
            (['exchange'], {
                'action': 'store',
                'help': 'name of exchange',
                'nargs': '?',
            }),
            (['currency'], {'action': 'store', 'help': ''}),
        ]

    @controller.expose(help='mark a fiat withdrawal as complete')
    def default(self):
        from gryphon.execution.controllers.withdraw import transaction_complete

        exchange_key = self.app.pargs.exchange
        currency = self.app.pargs.currency
        transaction_complete(exchange_key, currency)


class ScriptController(controller.CementBaseController):
    """This controller commands are 'stacked' onto the base controller."""

    class Meta:
        label = 'script'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'Script runner controller'
        arguments = [
            (['script_name'], {
                'action': 'store',
                'help': 'name of the script file in gryphon/execution/scripts',
                'nargs': '?',
            }),
            (['-X', '--execute'], {
                'action': 'store_true',
                'help': 'Run script in live mode, not test mode',
            }),
        ]

    @controller.expose(help='mark a fiat withdrawal as complete')
    def default(self):
        script_name = self.app.pargs.script_name
        execute = self.app.pargs.execute

        path = 'gryphon.execution.scripts.%s' % script_name

        function_name = 'main'  # All script files implement a main function.
        module = importlib.import_module(path)
        main_function = getattr(module, function_name)

        # This function should be renamed.
        script_arguments = config_helper.parse_extra_strategy_args(
            self.app.args.unknown_args,
        )

        main_function(script_arguments=script_arguments, execute=execute)


class GryphonFury(foundation.CementApp):
    class Meta:
        label = 'gryphon-fury'
        base_controller = GryphonFuryBaseController
        arguments_override_config = True

        argument_handler = 'my_argparse'

        # This is where the main config comes from.
        config_files = ['gryphon-fury.conf']

        handlers = [
            ModifiedArgparseArgumentHandler,
            OrderBookController,
            AuditController,
            ManualController,
            WithdrawController,
            WithdrawFiatController,
            TransactionCompleteController,
            ScriptController,
        ]


def main():
    with GryphonFury() as app:
        app.run()

if __name__ == '__main__':
    main()




"""
A library of functions having to do with getting revenue/profit and their several
varieties (open, realized, as a series).
"""

from collections import defaultdict
import copy
from more_itertools import chunked
from operator import attrgetter

from sqlalchemy import func
from sqlalchemy.orm import joinedload

from gryphon.lib.exchange.consts import Consts
import gryphon.lib.gryphonfury.fees as fees_lib
import gryphon.lib.gryphonfury.positions as positions
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Exchange as ExchangeData
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.money import Money
logger = get_logger(__name__)


def fast_revenue_in_period(db, start_time, end_time):
    """
    This achieves a faster revenue calculation than revenue_in_period through math and
    better queries. Previous function is roughly O(n^2) in the number of trades in the
    period and this one is likely O(logn) for a low constant.

    Refer to the helper function 
    get_revenue_in_period_given_positions_and_position_trades to see the heavy lifting
    """

    start_position, end_position = get_start_and_end_position(db, start_time, end_time)

    start_open_position_trades, end_open_position_trades = get_start_and_end_position_trades(db, start_time, end_time, start_position, end_position)

    revenue = get_revenue_in_period_given_positions_and_position_trades(
        db,
        start_time,
        end_time,
        start_position,
        end_position,
        start_open_position_trades,
        end_open_position_trades,
    )

    return revenue


def fast_profit_in_period(db, start_time, end_time):
    """
    Get profit in a period efficiently. If you are calculating revenue and profit at
    once, use fast_revenue_fees_profit_in_period for better performance.
    """

    revenue = fast_revenue_in_period(db, start_time, end_time)
    fees = fees_lib.get_matched_trading_fees_in_period(db, start_time, end_time)

    profit = revenue - fees

    return profit


def fast_revenue_fees_profit_in_period(db, start_time, end_time):
    """
    This function gives the same results as fast_(revenue|profit)_in_period but avoids
    duplicated computation between them for better performance.
    """

    # First get the start and end positions.
    start_position, end_position = get_start_and_end_position(db, start_time, end_time)

    # Now get the list of trades that created these positions.
    start_open_position_trades, end_open_position_trades = get_start_and_end_position_trades(db, start_time, end_time, start_position, end_position)

    revenue = get_revenue_in_period_given_positions_and_position_trades(
        db,
        start_time,
        end_time,
        start_position,
        end_position,
        start_open_position_trades,
        end_open_position_trades,
    )

    matched_fees = get_matched_fees_in_period_given_position_trades(
        db,
        start_time,
        end_time,
        start_open_position_trades,
        end_open_position_trades,
    )

    profit = revenue - matched_fees

    return revenue, matched_fees, profit


def get_matched_fees_in_period_given_position_trades(db, start_time, end_time, start_open_position_trades, end_open_position_trades):
    """
    This function is used for getting matched fees in a period efficiently if we
    already have the open positions and trades for those positions.

    It takes the start/ending positions for a period and the trades for those
    positions and finds the matched fees in that period, which the fees in the period
    plus fees on the open position at the beginning of the period minus fees on
    the open position at the end of the period.
    """

    start_position_fee = sum(
        [t.fee_in_usd for t in start_open_position_trades]
    )

    end_position_fee = sum(
        [t.fee_in_usd for t in end_open_position_trades]
    )

    unmatched_fees = fees_lib.get_all_fees_in_period_in_usd(db, start_time, end_time)
    matched_fees = unmatched_fees + start_position_fee - end_position_fee

    return matched_fees


def get_revenue_in_period_given_positions_and_position_trades(db, start_time, end_time, start_position, end_position, start_open_position_trades, end_open_position_trades):
    """
    This takes the start/ending positions for a period and the trades for those
    positions and finds the realized revenue in that period.

    The math is tough to write out in ascii but the idea is:
    revenue = total_asks_price - total_bids_price

    Because we only consider realized revenue in a period, the complexity of this
    function that we have to include the trades which created the open position at the
    beginning of this period, and remove the trades which created the final position
    in this period.
    """

    # Get the total price sum for all asks and bids in this period.
    asks, bids = get_total_asks_and_bids_prices_in_period_in_usd(
        db,
        start_time,
        end_time,
    )

    # Now determine the total price of creating these positions, so we can remove
    # these totals from asks or bids.
    start_position_price = sum(
        [t.price_in_usd for t in start_open_position_trades]
    )

    end_position_price = sum(
        [t.price_in_usd for t in end_open_position_trades]
    )

    # Add the price of the starting position to asks or bids, depending on whether
    # the position was long or short. We do this because this position was closed in
    # the period we are looking at now, so it's realized revenue belongs to this
    # period.
    if start_position > 0:
        bids = bids + start_position_price
    else:
        asks = asks + start_position_price

    # Remove the price of trades that contributed to our position at the end of the
    # period. We do this because this position will be closed in the next period, so
    # doesn't represent realized revenue in this period.
    if end_position > 0:
        bids = bids - end_position_price
    else:
        asks = asks - end_position_price

    # Having accounted for starting and ending positions, the calculation is
    # simply this:
    revenue = asks - bids

    return revenue


def get_start_and_end_position(db, start_time, end_time):
    """
    Helper function to get the positions at the beginning and end of a period
    efficiently.
    """

    start_position = positions.fast_position(
        db,
        end_time=start_time,
    )

    # We calculate end_position from the beginning of the day and then add the
    # start position for performance.
    end_position = positions.fast_position(
        db,
        start_time=start_time,
        end_time=end_time,
    )

    end_position = start_position + end_position

    return start_position, end_position


def get_start_and_end_position_trades(db, start_time, end_time, start_position, end_position):
    """
    Helper function to get the position trades at the start and end of a period
    efficiently.
    """

    start_open_position_trades = open_position_trades(
        start_position,
        db,
        start_time,
    )

    end_open_position_trades = open_position_trades(
        end_position,
        db,
        end_time,
    )

    return start_open_position_trades, end_open_position_trades


def get_total_asks_and_bids_prices_in_period_in_usd(db, start_time, end_time):
    totals = db.query(
            func.sum(Trade.price_in_usd),
            Order.order_type,
        )\
        .join(Order)\
        .filter(Order.actor == "Multi")\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .group_by(Order.order_type)\
        .all()

    # Default these to zero because we can have zero trades of a type in a small
    # period.
    bids = Money(0, 'USD')
    asks = Money(0, 'USD')

    for total in totals:
        order_type = total[1]
        price_in_usd = Money(total[0], 'USD')

        if order_type == Order.BID:
            bids = price_in_usd
        elif order_type == Order.ASK:
            asks = price_in_usd

    return asks, bids


### Slower functions below this line that still have certain uses ###


def profit_data(matched_trades, price_currency=None, volume_currency='BTC'):
    """
    Take a matched_trades object and construct profit, revenue, fees, and
    volume_currency_fees summary stats from it.
    """

    if not price_currency:
        price_currency = price_currency_for_trades(matched_trades)

    position = Money('0', volume_currency)

    revenue = Money('0', price_currency)
    fees = Money('0', price_currency)
    volume_currency_fees = Money('0', volume_currency)

    for trade in matched_trades:
        if trade.trade_type == Consts.ASK:
            position -= trade.volume
            revenue += trade.price_in_currency(price_currency)
        else:
            position += trade.volume
            revenue -= trade.price_in_currency(price_currency)

        # The USD equivalent of these fees get included in fees below, but we still
        # want to track them seperately.
        if trade.fee.currency == volume_currency:
            volume_currency_fees += trade.fee

        fees += trade.fee_in_currency(price_currency)

    if position != Money('0', volume_currency):
        raise ValueError("Matched Trades must sum up to 0 %s" % volume_currency)

    profit = revenue - fees

    return profit, revenue, fees, volume_currency_fees


def all_fees(trades, price_currency=None, volume_currency='BTC'):
    """
    Return the fiat equivalent fees for a list of trades and the exact BTC amount of any
    BTC fees (their USD value is still included in the total).
    """

    if not price_currency:
        price_currency = price_currency_for_trades(trades)

    all_fees = Money('0', price_currency)
    all_volume_currency_fees = Money('0', volume_currency)

    for t in trades:
        if t.fee.currency == volume_currency:
            all_volume_currency_fees += t.fee

        all_fees += t.fee_in_currency(price_currency)

    return all_fees, all_volume_currency_fees


def realized_pl(matched_trades, price_currency=None, volume_currency='BTC'):
    """
    Take a matched_trades object and return our realized profit in the price currency
    over those trades. We do this with the insight that for a matched_trades object,
    the position_delta in the volume currency will be zero, and the position delta in
    the price currency will be our realized profit.
    """

    realized_position = positions.position_delta(
        matched_trades,
        price_currency,
        volume_currency,
    )

    if realized_position[volume_currency] != Money('0', volume_currency):
        raise ValueError('Matched Trades must sum up to 0 %s' % volume_currency)

    realized_pl = realized_position['fiat']

    return realized_pl


def open_pl(open_position_trades, fundamental_value, price_currency=None, volume_currency='BTC'):
    """
    Take in an open_position_trades object and the current fundamental value, and use
    that to calculate our current unrealized profit on that position. This is done
    simply because the fiat position on those open_position_trades is the price we
    paid to open that position, and then the unrealized profit is our current 
    expectation of the value of that position (position * fundamental_value).
    """

    open_position = positions.position_delta(
        open_position_trades,
        price_currency,
        volume_currency=volume_currency,
    )

    fiat_position = open_position['fiat']
    volume_position = open_position[volume_currency]

    value_of_volume_position = fundamental_value * volume_position.amount

    open_pl = fiat_position + value_of_volume_position

    return open_pl


def copy_trade(trade):
    """
    Make a modifiable copy of a Trade object for use in split_trades.
    """
    new_trade = Trade(
        trade.trade_type,
        copy.copy(trade.price),
        copy.copy(trade.fee),
        copy.copy(trade.volume),
        trade.exchange_trade_id,
        None,
    )

    new_trade.time_created = trade.time_created

    # This is kinda janky: add a temporary property so we don't need to bring orders
    # into the copy. This never gets saved since these SQLAlchemy objects are never
    # attached to a session.
    new_trade._exchange_rate = trade.exchange_rate
    new_trade._exchange_name = trade.exchange_name
    new_trade._fundamental_value = trade.fundamental_value

    return new_trade


def split_trades(trades, price_currency='USD', volume_currency='BTC'):
    """
    This function creates the matched_trades and position_trades objects. Matched_trades
    is a list of pairs of semi-real trades, in which the first fake-trade represents a
    chunk of a real trade that opened a position, and the second fake-trade represents
    the chunk of another real trade that closed the first position. In this way the pair
    represents a complete loop of an open-and-closing of a position. This structure is
    very useful for several purposes.
    """

    # This is slow but safer for now, don't want to modify original array.
    trades_copy = []

    for t in trades:
        # We don't need orders here, so we could probaby use SimpleTrade.
        tc = copy_trade(t)
        trades_copy.append(tc)

    trades = trades_copy
    trades.sort(key=lambda t: t.time_created)

    total_volume = sum([t.volume for t in trades])
    total_price = sum([t.price_in_currency(price_currency) for t in trades])

    bids = []
    asks = []

    for t in trades:
        if t.trade_type == Consts.BID:
            bids.append(t)
        else:
            asks.append(t)

    matched_trades = []

    while bids and asks:
        if bids[0].volume < asks[0].volume:
            active = bids.pop(0)
            trades_to_match = asks
        else:
            active = asks.pop(0)
            trades_to_match = bids

        if active.volume > 0:
            matched_trades.append(active)

        volume_to_match = active.volume

        while volume_to_match > 0:
            match = trades_to_match[0]

            if match.volume <= volume_to_match:  # We totally match.
                volume_to_match -= match.volume
                match = trades_to_match.pop(0)  # Actually remove it.

                if match.volume > 0:
                    matched_trades.append(match)
            else:  # We match part of the opposite.
                orig_price = match.price
                orig_volume = match.volume

                # Split the partial match into 2 trades.
                fraction = volume_to_match / match.volume
                matched_partial_trade = copy_trade(match)
                matched_partial_trade.price = match.price * fraction
                matched_partial_trade.fee = match.fee * fraction
                matched_partial_trade.volume = volume_to_match

                old_fraction = (match.volume - volume_to_match) / match.volume
                match.price *= old_fraction
                match.fee *= old_fraction
                match.volume -= volume_to_match

                assert abs(match.price + matched_partial_trade.price - orig_price) < 1e-10
                assert abs(match.volume + matched_partial_trade.volume - orig_volume) < 1e-10

                if matched_partial_trade.volume > 0:
                    matched_trades.append(matched_partial_trade)

                volume_to_match = 0

    assert bids == [] or asks == []  # One should be empty.

    position_trades = bids + asks

    # Assert that our matched_trades are matched.
    matched_position = positions.position_delta(
        matched_trades,
        price_currency=price_currency,
        volume_currency=volume_currency,
    )

    assert matched_position[volume_currency] == Money('0', volume_currency)

    # Assert that our total price and volume haven't changed.
    new_total_volume = sum([t.volume for t in (matched_trades + position_trades)])
    new_total_price = sum([
        t.price_in_currency(price_currency) for t in (matched_trades + position_trades)
    ])

    assert abs(new_total_volume - total_volume) < 1e-10, \
        "%s != %s" % (new_total_volume, total_volume)

    assert abs(new_total_price - total_price) < 1e-10, \
        "%s != %s" % (new_total_price, total_price)

    return matched_trades, position_trades


def profit_units(matched_trades, price_currency=None):
    """
    Takes in a matched_trades object and returns a profit_units object. Profit units are
    simply all the steps that created realized profit in that list. E.g. a period with
    a 1 btc ask at $10 and a 0.1 btc bid at $9 will have a single profit unit with a
    value of $0.10.

    Usually we only use this function when we want to display the time series of profit
    in a given period. Other functions are faster for just getting the raw numbers.
    """

    if not price_currency:
        price_currency = price_currency_for_trades(matched_trades)

    profit_units = []

    for t1, t2 in chunked(matched_trades, 2):
        assert(t1.volume == t2.volume)

        if t1.trade_type == Consts.ASK and t2.trade_type == Consts.BID:
            ask = t1
            bid = t2
        elif t1.trade_type == Consts.BID and t2.trade_type == Consts.ASK:
            bid = t1
            ask = t2
        else:
            raise ValueError("matching trades are not opposite types")

        ask_price = ask.price_in_currency(price_currency)
        bid_price = bid.price_in_currency(price_currency)
        revenue = ask_price - bid_price

        ask_fee = ask.fee_in_currency(price_currency)
        bid_fee = bid.fee_in_currency(price_currency)
        fees = ask_fee + bid_fee

        profit = revenue - fees

        unit = {
            'time': max(t1.time_created, t2.time_created),
            'profit': profit,
            'revenue': revenue,
            'exchanges': [t1.exchange_name, t2.exchange_name],
        }

        profit_units.append(unit)

    return profit_units


def price_currency_for_trades(trades):
    """
    Simply takes a list of trades and extracts their currency.
    """

    if not trades:
        return "USD"

    currency = trades[0].price.currency

    # If our trades are in different currencies, convert them all to USD.
    if not all([(t.price.currency == currency) for t in trades]):
        currency = "USD"

    return currency


def open_position_trades(open_position_offset, db, start_time, volume_currency='BTC', strategy_actor='Multi'):
    """
    Find the last n trades from the previous day which opened the position we had
    before the day closed. We just scan backwards from the end of the day, finding
    trades of the correct type (ASK/BID) that add up to more volume than our position
    offset then we trim the last trade to the correct size so we have an exact set of
    trades.
    """

    if open_position_offset == Money('0', volume_currency):
        return []
    elif open_position_offset > Money('0', volume_currency):
        trade_type = Consts.BID
    else:
        trade_type = Consts.ASK

    open_position_trades = []

    while sum([t.volume for t in open_position_trades]) < abs(open_position_offset):
        query = db.query(Trade).join(Order)\
            .filter(Order.actor == strategy_actor)\
            .filter(Trade.trade_type == trade_type)\
            .filter(Trade.time_created < start_time)\
            .options(joinedload('order'))\
            .order_by(Trade.time_created.desc())

        if open_position_trades:
            # Filter out trades we already got.
            query = query.filter(
                ~Trade.unique_id.in_([t.unique_id for t in open_position_trades])
            )  

        trades_batch = query.limit(10).all()

        for trade in trades_batch:
            if (sum([t.volume for t in open_position_trades]) 
                    < abs(open_position_offset)):
                open_position_trades.append(trade)

    total_volume = sum([t.volume for t in open_position_trades])

    # This is the amount to trim off the last trade.
    volume_to_trim = total_volume - abs(open_position_offset)
    last_trade = open_position_trades.pop()

    # Copy it so we're not modifying a db object.
    last_trade = copy_trade(last_trade)
    volume_to_keep = last_trade.volume - volume_to_trim

    # Price and fee are absolutes, not per-bitcoin, so we need to trim them as well.
    ratio = volume_to_keep / last_trade.volume
    last_trade.volume *= ratio
    last_trade.price *= ratio
    last_trade.fee *= ratio

    open_position_trades.append(last_trade)

    # Flip these around so oldest are first like the rest of our trades.
    open_position_trades.reverse()

    return open_position_trades


def exchange_profit_participation(profit_units):
    """
    Given a list of profit units, returns a dictionary with the percentage of the total
    profit that was gained from trading on each exchange.
    """
    exchange_absolute_profits = defaultdict(lambda: Money(0, 'USD'))

    # This is multiplied by two because every dollar of profit is attributed to two
    # exchanges, therefore to get percentages that add up to 100 we have to count every
    # dollar twice. It works.
    total_profit = sum([unit['profit'] for unit in profit_units])

    for unit in profit_units:
        for exchange_name in unit['exchanges']:
            exchange_profit = exchange_absolute_profits[exchange_name]

            exchange_profit += unit['profit'] / 2

            exchange_absolute_profits[exchange_name] = exchange_profit

    exchange_percentage_profits = {}

    for exchange_name, exchange_profit in exchange_absolute_profits.items():
        exchange_percentage_profits[exchange_name] = exchange_profit / total_profit

    return exchange_percentage_profits


def rsb(revenue, total_volume):
    real_spread_per_bitcoin = None

    if total_volume > 0:
        real_spread_per_bitcoin = (revenue.amount) / total_volume.amount

    return real_spread_per_bitcoin


def fsb(profit, total_volume):
    fee_spread_per_bitcoin = None

    if total_volume > 0:
        fee_spread_per_bitcoin = (profit.amount) / total_volume.amount

    return fee_spread_per_bitcoin



import json

from delorean import Delorean
from twisted.internet import defer

import gryphon.data_service.consts as consts
from gryphon.data_service.pollers.request_poller import RequestPoller
import gryphon.data_service.util as util


class VolumePoller(RequestPoller):
    @property
    def volume_key(self):
        return '%s_volume' % self.exchange_name.lower()

    @property
    def heartbeat_key(self):
        return '%s_volume_heartbeat' % self.exchange_name.lower()

    @defer.inlineCallbacks
    def start(self):
        self.poll_time = 55
        binding_key = '%s.exchange_volume.tinker' % self.exchange_name.lower()

        self.producer = yield util.setup_producer(
            consts.EXCHANGE_VOLUME_QUEUE,
            binding_key,
        )

        super(VolumePoller, self).start()

    def parse_volume_json(self, volume_json):
        """ Parses incoming json from exchange into a string number """
        raise NotImplementedError

    def parse_response(self, response):
        """ Parses the volume response from the exchange. """

        volume = self.parse_volume_json(response)
        raw_timestamp = Delorean().epoch

        if volume and self.producer:
            new_volume = {
                'timestamp': raw_timestamp,
                'exchange_name': self.exchange_name,
                'volume': volume,
            }

            new_volume_string = json.dumps(new_volume, ensure_ascii=False)
            self.producer.publish_message(new_volume_string)



# -*- coding: utf-8 -*-
import subprocess
import time

import pika

from gryphon.lib.logger import get_logger

logger = get_logger('emerald-havoc-consumer')


class QueueConsumerException(Exception):
    pass


class QueueConsumer(object):
    """
    This is an example consumer that will handle unexpected interactions with RabbitMQ
    such as channel and connection closures.

    If RabbitMQ closes the connection, it will reopen it. You should look at the output,
    as there are limited reasons why the connection may be closed, which usually are
    tied to permission related issues or socket timeouts.

    If the channel is closed, it will indicate a problem with one of the commands that
    were issued and that should surface in the output as well.
    """

    def __init__(self, amqp_url, consumer_func, consumer_func_args, exchange, exchange_type, binding_key, queue_name):
        """
        Create a new instance of the consumer class, passing in the AMQP URL used to
        connect to RabbitMQ.

        :param str amqp_url: The AMQP url to connect with
        """

        self._connection = None
        self._channel = None
        self._closing = False
        self._consumer_tag = None
        self._url = amqp_url

        self.consumer_func = consumer_func
        self.consumer_func_args = consumer_func_args

        self.exchange = exchange
        self.exchange_type = exchange_type
        self.binding_key = binding_key
        self.queue_name = queue_name

    def connect(self):
        """
        This method connects to RabbitMQ, returning the connection handle.  When the
        connection is established, the on_connection_open method will be invoked by
        pika.

        :rtype: pika.SelectConnection
        """

        logger.info('Connecting to %s', self._url)

        return pika.SelectConnection(
            pika.URLParameters(self._url),
            self.on_connection_open,
            stop_ioloop_on_close=False,
        )

    def close_connection(self):
        """This method closes the connection to RabbitMQ."""

        logger.info('Closing connection')
        self._connection.close()

    def on_connection_closed(self, connection, reply_code, reply_text):
        """
        This method is invoked by pika when the connection to RabbitMQ is closed
        unexpectedly. Since it is unexpected, we will reconnect to RabbitMQ if it
        disconnects.

        :param pika.connection.Connection connection: The closed connection obj
        :param int reply_code: The server provided reply_code if given
        :param str reply_text: The server provided reply_text if given
        """

        self._channel = None

        if self._closing:
            self._connection.ioloop.stop()
        else:
            logger.warning(
                'Connection closed, reopening in 5 seconds: (%s) %s',
                reply_code,
                reply_text,
            )

            self._connection.add_timeout(5, self.reconnect)

    def on_connection_open(self, unused_connection):
        """
        This method is called by pika once the connection to RabbitMQ has been
        established. It passes the handle to the connection object in case we need it,
        but in this case, we'll just mark it unused.

        :type unused_connection: pika.SelectConnection
        """

        logger.info('Connection opened')
        logger.info('Adding connection close callback')

        self._connection.add_on_close_callback(self.on_connection_closed)
        self._connection.channel(on_open_callback=self.on_channel_open)

    def reconnect(self):
        """
        Will be invoked by the IOLoop timer if the connection is closed. See the
        on_connection_closed method.
        """

        # This is the old connection IOLoop instance, stop its ioloop.
        self._connection.ioloop.stop()

        if not self._closing:
            # Create a new connection.
            self._connection = self.connect()

            # There is now a new connection, needs a new ioloop to run.
            self._connection.ioloop.start()

    def on_channel_closed(self, channel, reply_code, reply_text):
        """
        Invoked by pika when RabbitMQ unexpectedly closes the channel. Channels are
        usually closed if you attempt to do something that violates the protocol, such
        as re-declare an exchange or queue with different parameters. In this case,
        we'll close the connection to shutdown the object.

        :param pika.channel.Channel: The closed channel
        :param int reply_code: The numeric reason the channel was closed
        :param str reply_text: The text reason the channel was closed
        """

        logger.warning(
            'Channel %i was closed: (%s) %s',
            channel,
            reply_code,
            reply_text,
        )

        self._connection.close()

    def on_channel_open(self, channel):
        """
        This method is invoked by pika when the channel has been opened. The channel
        object is passed in so we can make use of it.

        Since the channel is now open, we'll declare the exchange to use.

        :param pika.channel.Channel channel: The channel object
        """

        logger.info('Channel opened')

        self._channel = channel

        logger.info('Adding channel close callback')

        self._channel.add_on_close_callback(self.on_channel_closed)

        logger.info('Declaring exchange %s', self.exchange)

        self._channel.exchange_declare(
            self.on_exchange_declareok,
            self.exchange,
            self.exchange_type,
        )

    def on_exchange_declareok(self, unused_frame):
        """
        Invoked by pika when RabbitMQ has finished the Exchange.Declare RPC command.

        :param pika.Frame.Method unused_frame: Exchange.DeclareOk response frame
        """

        logger.info('Exchange declared')
        logger.info('Declaring queue %s', self.queue_name)

        self._channel.queue_declare(
            self.on_queue_declareok,
            self.queue_name,
            durable=True,
        )

    def on_queue_declareok(self, method_frame):
        """
        Method invoked by pika when the Queue.Declare RPC call made in setup_queue has
        completed. In this method we will bind the queue and exchange together with the
        routing key by issuing the Queue.Bind RPC command. When this command is complete
        , the on_bindok method will be invoked by pika.

        :param pika.frame.Method method_frame: The Queue.DeclareOk frame
        """

        logger.info(
            'Binding %s to %s with %s',
            self.exchange,
            self.queue_name,
            self.binding_key,
        )

        self._channel.queue_bind(
            self.on_bindok,
            self.queue_name,
            self.exchange,
            self.binding_key,
        )

    def on_consumer_cancelled(self, method_frame):
        """
        Invoked by pika when RabbitMQ sends a Basic.Cancel for a consumer receiving
        messages.

        :param pika.frame.Method method_frame: The Basic.Cancel frame
        """

        logger.info(
            'Consumer was cancelled remotely, shutting down: %r',
            method_frame,
        )

        if self._channel:
            self._channel.close()

    def acknowledge_message(self, delivery_tag):
        """
        Acknowledge the message delivery from RabbitMQ by sending a Basic.Ack RPC
        method for the delivery tag.

        :param int delivery_tag: The delivery tag from the Basic.Deliver frame
        """

        self._channel.basic_ack(delivery_tag)

    def on_message(self, unused_channel, basic_deliver, properties, body, attempt=1):
        """
        Invoked by pika when a message is delivered from RabbitMQ. The channel is
        passed for your convenience. The basic_deliver object that is passed in carries
        the exchange, routing key, delivery tag and a redelivered flag for the message.
        The properties passed in is an instance of BasicProperties with the message
        properties and the body is the message that was sent.

        :param pika.channel.Channel unused_channel: The channel object
        :param pika.Spec.Basic.Deliver: basic_deliver method
        :param pika.Spec.BasicProperties: properties
        :param str|unicode body: The message body
        """

        # https://www.rabbitmq.com/tutorials/tutorial-two-python.html
        # see Message Acknowledgement.
        # We now only heartbeat on a successful consumer_func call.
        try:
            self.consumer_func(body, self.consumer_func_args)

            heartbeat_key = self.queue_name.lower()

            subprocess.call([
                'touch',
                'monit/heartbeat/%s_consumer_success_heartbeat.txt' % heartbeat_key,
            ])

            self.acknowledge_message(basic_deliver.delivery_tag)
        except:
            logger.exception('Attempt %s: FAILED MESSAGE: %s' % (attempt, body[:100]))

            if attempt > 600:
                raise QueueConsumerException(
                    'Queue: %s has had too many failures' % self.queue_name.lower(),
                )

            # We are going to sleep for 1 second before retrying.
            time.sleep(1)

            self.on_message(
                unused_channel,
                basic_deliver,
                properties,
                body,
                attempt=attempt + 1,
            )

    def on_cancelok(self, unused_frame):
        """
        This method is invoked by pika when RabbitMQ acknowledges the cancellation of
        a consumer. At this point we will close the channel. This will invoke the
        on_channel_closed method once the channel has been closed, which will in-turn
        close the connection.

        :param pika.frame.Method unused_frame: The Basic.CancelOk frame
        """

        logger.info('RabbitMQ acknowledged the cancellation of the consumer')

        self.close_channel()

    def stop_consuming(self):
        """
        Tell RabbitMQ that you would like to stop consuming by sending the Basic.Cancel
        RPC command.
        """

        if self._channel:
            logger.info('Sending a Basic.Cancel RPC command to RabbitMQ')
            self._channel.basic_cancel(self.on_cancelok, self._consumer_tag)

    def start_consuming(self):
        """
        This method sets up the consumer by first calling add_on_cancel_callback
        so that the object is notified if RabbitMQ cancels the consumer. It then
        issues the Basic.Consume RPC command which returns the consumer tag that is
        used to uniquely identify the consumer with RabbitMQ. We keep the value to use
        it when we want to cancel consuming. The on_message method is passed in as a
        callback pika will invoke when a message is fully received.
        """

        logger.info('Issuing consumer related RPC commands')
        logger.info('Adding consumer cancellation callback')

        self._channel.add_on_cancel_callback(self.on_consumer_cancelled)

        self._consumer_tag = self._channel.basic_consume(
            self.on_message,
            self.queue_name,
        )

    def on_bindok(self, unused_frame):
        """
        Invoked by pika when the Queue.Bind method has completed. At this point we
        will start consuming messages by calling start_consuming which will invoke the
        needed RPC commands to start the process.

        :param pika.frame.Method unused_frame: The Queue.BindOk response frame
        """

        logger.info('Queue bound')
        self.start_consuming()

    def close_channel(self):
        """
        Call to close the channel with RabbitMQ cleanly by issuing the Channel.Close
        RPC command.
        """

        logger.info('Closing the channel')
        self._channel.close()

    def run(self):
        """
        Run the example consumer by connecting to RabbitMQ and then starting the IOLoop
        to block and allow the SelectConnection to operate.
        """

        self._connection = self.connect()
        self._connection.ioloop.start()

    def stop(self):
        """
        Cleanly shutdown the connection to RabbitMQ by stopping the consumer with
        RabbitMQ. When RabbitMQ confirms the cancellation, on_cancelok will be invoked
        by pika, which will then closing the channel and connection. The IOLoop is
        started again because this method is invoked when CTRL-C is pressed raising a
        KeyboardInterrupt exception. This exception stops the IOLoop which needs to be
        running for pika to communicate with RabbitMQ. All of the commands issued prior
        to starting the IOLoop will be buffered but not processed.
        """

        logger.info('Stopping')

        self._closing = True
        self.stop_consuming()
        self._connection.ioloop.start()

        logger.info('Stopped')




"""
Tests for the gryphon.lib configuration library.
"""

import pyximport; pyximport.install()
import unittest
import mock
import sure

from cdecimal import Decimal

from gryphon.lib import configuration
from gryphon.lib.money import Money


TRIVIAL_CONFIG_FILE = """\
[strategy]
midpoint: 0.001
"""

SIMPLE_CONFIG_FILE = """\
[strategy]
midpoint: 0.001
quote_depth = BTC 20
use_gds: yes
primary_exchange_name: coinbase_btc_usd
"""

MULTISECTION_CONFIG_FILE = """\
[platform]
audit: no

[strategy]
tick_sleep: 1
"""

SUBSECTION_CONFIG_FILE = """\
[platform]
audit: no

[strategy]
tick_sleep: 1

[strategy:midpoint_weights]
coinbase_btc_usd: 0.5
bitstamp_btc_usd: 0.5
"""


class TestConfiguration(unittest.TestCase):
    def test_parse_list_simple(self):
        value = 'bitstamp'

        output = configuration.parse_configurable_as_list(value)
      
        len(output).should.equal(1)
        output[0].should.equal('bitstamp')

    def test_parse_list_more(self):
        value = 'bitstamp,coinbase,kraken'

        output = configuration.parse_configurable_as_list(value)
      
        len(output).should.equal(3) 
        output[0].should.equal('bitstamp')
        output[1].should.equal('coinbase')
        output[2].should.equal('kraken')

    def test_parse_list_simple_trailing_comma(self):
        value = 'bitstamp,'

        output = configuration.parse_configurable_as_list(value)
      
        len(output).should.equal(1) 
        output[0].should.equal('bitstamp')

    def test_parse_list_more_trailing_comma(self):
        value = 'bitstamp,coinbase,kraken,'

        output = configuration.parse_configurable_as_list(value)
      
        len(output).should.equal(3) 
        output[0].should.equal('bitstamp')
        output[1].should.equal('coinbase')
        output[2].should.equal('kraken')

    def test_parse_list_many_commas(self):
        value = ',bitstamp,coinbase,,,kraken,,,,'

        output = configuration.parse_configurable_as_list(value)
      
        len(output).should.equal(3) 
        output[0].should.equal('bitstamp')
        output[1].should.equal('coinbase')
        output[2].should.equal('kraken')

    def test_configurable_value_money(self):
        value = 'BTC 0.1'

        output = configuration.parse_configurable_value(value)

        output.should.equal(Money('0.1', 'BTC'))

    def test_configurable_value_money_2(self):
        value = 'ETH 0.1'

        output = configuration.parse_configurable_value(value)

        output.should.equal(Money('0.1', 'ETH'))

    def test_configurable_value_bad_money(self):
        value = '1000 USD'

        output = configuration.parse_configurable_value(value)

        output.should.equal('1000 USD')

    def test_configurable_value_number(self):
        value = '133120102'

        output = configuration.parse_configurable_value(value)

        output.should.equal(Decimal('133120102'))

    def test_configurable_value_number_2(self):
        value = '0'

        output = configuration.parse_configurable_value(value)

        output.should.equal(Decimal('0'))

    def test_configurable_value_bool(self):
        value = 'yes'

        output = configuration.parse_configurable_value(value)

        output.should.equal(True)

    def test_configurable_value_bool_2(self):
        value = 'no'

        output = configuration.parse_configurable_value(value)

        output.should.equal(False)

    def test_configurable_value_bool_3(self):
        value = True

        output = configuration.parse_configurable_value(value)

        output.should.equal(True)

    def test_configurable_value_bool_4(self):
        value = False

        output = configuration.parse_configurable_value(value)

        output.should.equal(False)

    def test_configurable_value_string(self):
        value = 'tornado'

        output = configuration.parse_configurable_value(value)

        output.should.equal('tornado')

    def test_parse_sections_trivial(self):
        parser = configuration._get_parser_for_string_config(TRIVIAL_CONFIG_FILE)

        parsed = configuration.parse_sections(parser)

        len(parsed.keys()).should.equal(1)
        parsed.keys().should.equal(['strategy'])
        parsed['strategy'].should.equal({'midpoint': Decimal('0.001')})

    def test_parse_sections_simple(self):
        parser = configuration._get_parser_for_string_config(SIMPLE_CONFIG_FILE)

        parsed = configuration.parse_sections(parser)

        len(parsed.keys()).should.equal(1)
        parsed.keys().should.equal(['strategy'])
        parsed['strategy']['midpoint'].should.equal(Decimal('0.001'))
        parsed['strategy']['quote_depth'].should.equal(Money('20', 'BTC'))
        parsed['strategy']['use_gds'].should.equal(True)
        parsed['strategy']['primary_exchange_name'].should.equal('coinbase_btc_usd')

    def test_parse_sections_multi_section(self):
        parser = configuration._get_parser_for_string_config(MULTISECTION_CONFIG_FILE)

        parsed = configuration.parse_sections(parser)

        len(parsed.keys()).should.equal(2)
        parsed.keys().should.equal(['platform', 'strategy'])
        parsed['strategy']['tick_sleep'].should.equal(Decimal('1'))
        parsed['platform']['audit'].should.equal(False)

    def test_parse_sections_subsection(self):
        parser = configuration._get_parser_for_string_config(SUBSECTION_CONFIG_FILE)

        parsed = configuration.parse_sections(parser)

        len(parsed.keys()).should.equal(2)
        parsed.keys().should.equal(['platform', 'strategy'])
        parsed['strategy']['tick_sleep'].should.equal(Decimal('1'))
        parsed['platform']['audit'].should.equal(False)

        parsed['strategy']['midpoint_weights'].keys().should.equal([
            'coinbase_btc_usd',
            'bitstamp_btc_usd',
        ])

        parsed['strategy']['midpoint_weights']['coinbase_btc_usd']\
            .should.equal(Decimal('0.5'))

        parsed['strategy']['midpoint_weights']['bitstamp_btc_usd']\
            .should.equal(Decimal('0.5'))





"""
This script sets up an exchange, or set of exchanges, for trading in your gryphon
database.

To trade with an exchange in Gryphon, we need to start a ledger of trades and
transactions for that exchange. In the current database, a ledger is simply an entry
in the Exchange table that is referenced by entries in the Order, Trade, and
Transaction tables.

For us to start trading we need to create that entry in the Exchange table, and add two
dummy transactions that represent the starting balance of the exchange.

Usage:
    gryphon-execute script initialize_exchange_ledger
        --exchanges [comma-separated list of exchange pairs, e.g. bitstamp_btc_usd]
        [--execute]
"""

import pyximport; pyximport.install()

import os

from gryphon.lib import configuration
from gryphon.lib import session
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.models.exchange import Exchange as ExchangeData
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money


ALREADY_INITIALIZED_ERR_MESSAGE = """\
Unable to initialize %s, there is already a db entry for that exchange name.\
"""

NO_API_CREDENTIALS_ERR_MESSAGE = """\
Could not initialize %s because we couldn't find API credentials for it.\
"""

UNKNOWN_ERR_MESSAGE = """\
Could not initialize %s, for an unexpected reason.\
"""

INITIALIZATION_TX_NOTES = """Initializing the db's %s balance"""


def initialize_exchange_ledger(db, wrapper_obj):
    db_obj = None

    try:
        db_obj = wrapper_obj.exchange_account_db_object(db)
    except AssertionError:
        # This is fine, the above line is supposed to fail.
        pass
    finally:
        if db_obj is not None:
            print ALREADY_INITIALIZED_ERR_MESSAGE % wrapper_obj.name
            return

    # Create the entry in the Exchange table.
    db_obj = ExchangeData(wrapper_obj.name)

    # Proceed to creating initializing transactions.
    price_currency = wrapper_obj.currency
    vol_currency = wrapper_obj.volume_currency

    try:
        balance = wrapper_obj.get_balance()
    except KeyError as e:
        print NO_API_CREDENTIALS_ERR_MESSAGE % wrapper_obj.name
        print e
        return
    except Exception as e:
        print UNKNOWN_ERR_MESSAGE % wrapper_obj.name
        print e
        return

    price_currency = wrapper_obj.currency
    vol_currency = wrapper_obj.volume_currency

    price_currency_balance = balance[price_currency]
    price_tx_details = {
        'notes': INITIALIZATION_TX_NOTES % price_currency,
    }

    vol_currency_balance = balance[vol_currency]
    vol_tx_details = {
        'notes': INITIALIZATION_TX_NOTES % vol_currency,
    }

    price_currency_tx = Transaction(
        Transaction.DEPOSIT,
        Transaction.IN_TRANSIT,
        price_currency_balance,
        db_obj,
        transaction_details=price_tx_details,
        fee=Money('0', price_currency),
    )

    vol_currency_tx = Transaction(
        Transaction.DEPOSIT,
        Transaction.IN_TRANSIT,
        vol_currency_balance,
        db_obj,
        transaction_details=vol_tx_details,
        fee=Money('0', vol_currency),
    )

    db.add(db_obj)
    db.add(price_currency_tx)
    db.add(vol_currency_tx)

    price_currency_tx.complete()
    vol_currency_tx.complete()


def main(script_arguments, execute):
    exchanges = configuration.parse_configurable_as_list(script_arguments['exchanges'])

    db = session.get_a_trading_db_mysql_session()

    for exchange_name in exchanges:
        exchange_wrapper = exchange_factory.make_exchange_from_key(exchange_name)
        initialize_exchange_ledger(db, exchange_wrapper)

    if execute is True:
        db.commit()




# -*- coding: utf-8 -*-
import json
import os

from delorean import Delorean
from twisted.internet import defer
from twisted.python import log

from gryphon.data_service.pollers.request_poller import RequestPoller


class OpenExchangeRatePoller(RequestPoller):
    def __init__(self):
        app_id = os.environ['EXCHANGE_RATE_APP_ID']
        self.url = 'https://openexchangerates.org/api/latest.json?app_id=%s&show_alternative=true' % app_id
        self.poll_time = 60
        self.heartbeat_key = 'open_exchange_rate_poller_heartbeat'

    @defer.inlineCallbacks
    def parse_response(self, resp_obj):
        ts = Delorean().epoch

        exchange_rate_string = json.dumps({
            'timestamp': ts,
            'exchange_rates': resp_obj,
        })

        if resp_obj:
            yield self.redis.set('emerald_havoc_exchange_rates', exchange_rate_string)

            log.msg('[Exchange Rate] Successful @ %s' % ts)
        else:
            log.msg('[Exchange Rage] Unsuccessful @ %s' % ts)




import pyximport; pyximport.install()

import os
import unittest
import sure

from cdecimal import Decimal

from gryphon.lib.metrics import midpoint
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.money import Money
from gryphon.lib.exchange.coinbase_btc_usd import CoinbaseBTCUSDExchange

class TestMidpoint(unittest.TestCase):
    def setUp(self):
        self.empty_book = {
            'bids': [],
            'asks': [],
        }

        self.exchange = CoinbaseBTCUSDExchange()

    def basic_book(self,price_currency='USD', vol_currency='BTC'):
        def price(val):
            return Money(val, price_currency)

        def vol(val):
            return Money(val, vol_currency)

        bids = [Order(price('249'), vol('10'), self.exchange, Order.BID)]
        asks = [Order(price('251'), vol('10'), self.exchange, Order.ASK)]

        return {'bids': bids, 'asks': asks}

    def bigger_book(self, price_currency='USD', vol_currency='BTC'):
        def price(value):
            return Money(value, price_currency)

        def vol(value):
            return Money(value, vol_currency)

        bids = [
            Order(price('249'), vol('10'), self.exchange, Order.BID),
            Order(price('248'), vol('10'), self.exchange, Order.BID)
        ]

        asks = [
            Order(price('251'), vol('10'), self.exchange, Order.ASK),
            Order(price('260'), vol('10'), self.exchange, Order.ASK),
        ]

        return {'bids': bids, 'asks': asks}

    def test_empty(self):
        midpoint.get_midpoint_from_orderbook.when.called_with(self.empty_book)\
            .should.throw(midpoint.OrderbookSizeException)

    def test_basic(self):
        result = midpoint.get_midpoint_from_orderbook(self.basic_book())
        result.should.equal(Money('250', 'USD'))

    def test_basic_non_btc(self):
        result = midpoint.get_midpoint_from_orderbook(
            self.basic_book(vol_currency='ETH'),
        )

        result.should.equal(Money('250', 'USD'))

    def test_basic_crypto_crypto(self):
        result = midpoint.get_midpoint_from_orderbook(
            self.basic_book(price_currency='BTC', vol_currency='ETH'),
        )

        result.should.equal(Money('250', 'BTC'))

    def test_basic_with_depth(self):
        result = midpoint.get_midpoint_from_orderbook(
            self.basic_book(price_currency='BTC', vol_currency='ETH'),
            depth=Money('0.5', 'ETH'),
        )

        result.should.equal(Money('250', 'BTC'))

    def test_basic_with_bad_depth(self):
        result = midpoint.get_midpoint_from_orderbook.when.called_with(
            self.basic_book(price_currency='BTC', vol_currency='ETH'),
            depth=Money('0.5', 'BTC'),
        ).should.throw(midpoint.OrderbookSizeException)

    def test_bigger(self):
        result = midpoint.get_midpoint_from_orderbook(self.basic_book())
        result.should.equal(Money('250', 'USD'))

    def test_bigger_with_depth(self):
        result = midpoint.get_midpoint_from_orderbook(
            self.bigger_book(),
            depth=Money('20', 'BTC'),
        )

        result.should.equal(Money('252', 'USD'))



import pyximport; pyximport.install()

import logging
import os
import unittest
import sure
import mock

logger = logging.getLogger(__name__)


class ExchangePublicMethodsTests(object):
    def test_orderbook(self):
        book = self.exchange.get_orderbook()

        assert len(book['bids']) > 10
        assert len(book['asks']) > 10

    def test_ticker(self):
        ticker = self.exchange.get_ticker()

        assert all([key in ticker for key in ('high', 'low', 'last', 'volume')])




from Crypto.Cipher import AES
import base64
import os
import binascii


def encrypt(plain_text, secret_key):
    cipher = AES.new(secret_key)
    plain_text = plain_text + (" " * (16 - (len(plain_text) % 16)))
    return binascii.hexlify(cipher.encrypt(plain_text))
    
def decrypt(encrypted_text, secret_key):
    cipher = AES.new(secret_key)
    return cipher.decrypt(binascii.unhexlify(encrypted_text)).rstrip()



"""Initial migration

Revision ID: 2c7cc7d09efe
Revises: None
Create Date: 2019-03-18 17:00:23.187262

"""

# revision identifiers, used by Alembic.
revision = '2c7cc7d09efe'
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('exchange_volume',
    sa.Column('exchange_volume_id', sa.Integer(), nullable=False),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('exchange', sa.Unicode(length=64), nullable=False),
    sa.Column('timestamp', sa.DateTime(), nullable=False),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.Column('exchange_volume', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.PrimaryKeyConstraint('exchange_volume_id')
    )
    op.create_table('orderbook',
    sa.Column('orderbook_id', sa.Integer(), nullable=False),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('exchange', sa.Unicode(length=64), nullable=False),
    sa.Column('timestamp', sa.DateTime(), nullable=True),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.Column('bids', sa.UnicodeText(length=2147483648), nullable=True),
    sa.Column('asks', sa.UnicodeText(length=2147483648), nullable=True),
    sa.PrimaryKeyConstraint('orderbook_id')
    )
    op.create_table('trade',
    sa.Column('trade_id', sa.Integer(), nullable=False),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('exchange', sa.Unicode(length=64), nullable=False),
    sa.Column('timestamp', sa.DateTime(), nullable=False),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.Column('exchange_trade_id', sa.Unicode(length=64), nullable=True),
    sa.Column('source', sa.Unicode(length=64), nullable=False),
    sa.Column('price', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('price_currency', sa.Unicode(length=3), nullable=True),
    sa.Column('volume', sa.Numeric(precision=20, scale=10), nullable=True),
    sa.Column('volume_currency', sa.Unicode(length=3), nullable=True),
    sa.PrimaryKeyConstraint('trade_id'),
    sa.UniqueConstraint('exchange','exchange_trade_id', name='uniq_exchange_exchange_trade_id')
    )

    op.create_index('idx_timestamp_exchange', 'exchange_volume', ['timestamp', 'exchange'])

    op.create_index('idx_timestamp_exchange', 'orderbook', ['timestamp', 'exchange'])
    op.create_index('idx_timestamp_exchange', 'trade', ['timestamp', 'exchange'])
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('trade')
    op.drop_table('orderbook')
    op.drop_table('exchange_volume')
    ### end Alembic commands ###



"""
Tests for gryphon.lib.arbitrage.

TODO:
  - Add more tests using full real orderbooks, likely with cassettes.
  - Use mock or some other approach to allow us to run tests that involve forex
    conversion without requireing an OXR account to do so. This is important so that
    we can test e.g. detecting arbitrage between USD and CAD priced orderbooks.
"""

import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

import unittest
import sure

from cdecimal import Decimal, ROUND_TRUNC

from gryphon.lib import arbitrage
from gryphon.lib.models.emeraldhavoc.orderbook import Orderbook
from gryphon.lib.money import Money
from gryphon.lib.exchange.bitstamp_btc_usd import BitstampBTCUSDExchange
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.exchange.itbit_btc_usd import ItbitBTCUSDExchange


class TestArbLib(unittest.TestCase):
    def setUp(self):
        self.itbit = ItbitBTCUSDExchange()
        self.bitstamp = BitstampBTCUSDExchange()

        self.itbit.market_order_fee = 0
        self.bitstamp.market_order_fee = 0

        self.trivial = {'bids': [], 'asks': []}

    def tearDown(self):
        pass

    def basic_ob_1(self, price_currency='USD', vol_currency='BTC'):
        ex = self.itbit

        ex.currency = price_currency
        ex.volume_currency = vol_currency

        volume = Money('1', vol_currency)

        bids2 = [
            Order(Money('598', price_currency), volume, ex, Consts.BID),
            Order(Money('550', price_currency), volume, ex, Consts.BID),
        ]

        asks2 = [
            Order(Money('599', price_currency), volume, ex, Consts.ASK),
            Order(Money('650', price_currency), volume, ex, Consts.ASK),
        ]

        return {'bids': bids2, 'asks': asks2}

    def basic_ob_2(self, price_currency='USD', vol_currency='BTC'):
        ex = self.bitstamp
        volume = Money('1', vol_currency)

        ex.currency = price_currency
        ex.volume_currency = vol_currency

        bids1 = [
            Order(Money('600', price_currency), volume, ex, Consts.BID),
            Order(Money('550', price_currency), volume, ex, Consts.BID),
        ]

        asks1 = [
            Order(Money('601', price_currency), volume, ex, Consts.ASK),
            Order(Money('650', price_currency), volume, ex, Consts.ASK),
        ]

        return {'bids': bids1, 'asks': asks1}

    def more_ob_1(self, price_currency='USD', vol_currency='BTC'):
        ex = self.bitstamp
        volume = Money('1', vol_currency)

        ex.currency = price_currency
        ex.volume_currency = vol_currency

        bids3 = [
            Order(Money('600', price_currency), volume, ex, Consts.BID),
            Order(Money('550', price_currency), volume, ex, Consts.BID),
        ]

        asks3 = [
            Order(Money('601', price_currency), volume, ex, Consts.ASK),
            Order(Money('650', price_currency), volume, ex, Consts.ASK),
        ]

        return {'bids': bids3, 'asks': asks3}

    def more_ob_2(self, price_currency='USD', vol_currency='BTC'):
        ex = self.itbit
        volume_1 = Money('1', vol_currency)
        volume_2 = Money('0.5', vol_currency)

        ex.currency = price_currency
        ex.volume_currency = vol_currency

        bids4 = [
            Order(Money('598', price_currency), volume_1, ex, Consts.BID),
            Order(Money('550', price_currency), volume_1, ex, Consts.BID),
        ]

        asks4 = [
            Order(Money('599', price_currency), volume_2, ex, Consts.ASK),
            Order(Money('599.5', price_currency), volume_2, ex, Consts.ASK),
            Order(Money('650', price_currency), volume_1, ex, Consts.ASK),
        ]


        return {'bids': bids4, 'asks': asks4}

    def real_orderbook_1(self, price_currency='USD', vol_currency='BTC'):
        ex = self.bitstamp

        ex.currency = price_currency
        ex.volume_currency = vol_currency

        def price(literal):
            return Money(literal, price_currency)

        def vol(literal):
            return Money(literal, vol_currency)

        bids = [
            Order(price('604.31'), vol('4.20600000'), ex, Consts.BID),
            Order(price('604.30'), vol('3.00000000'), ex, Consts.BID),
            Order(price('604.25'), vol('2.76500000'), ex, Consts.BID),
            Order(price('604.24'), vol('3.43900000'), ex, Consts.BID),
            Order(price('604.00'), vol('1.82278003'), ex, Consts.BID),
            Order(price('603.98'), vol('4.83394284'), ex, Consts.BID),
            Order(price('603.75'), vol('20.00000000'), ex, Consts.BID),
            Order(price('603.61'), vol('4.57000000'), ex, Consts.BID),
            Order(price('603.51'), vol('0.01615431'), ex, Consts.BID),
            Order(price('603.38'), vol('4.58980000'), ex, Consts.BID),
        ]

        asks = [
            Order(price('605.37'), vol('0.82039084'), ex, Consts.ASK),
            Order(price('605.41'), vol('1.07069908'), ex, Consts.ASK),
            Order(price('605.47'), vol('1.01762788'), ex, Consts.ASK),
            Order(price('605.50'), vol('10.00000000'), ex, Consts.ASK),
            Order(price('606.19'), vol('16.93200000'), ex, Consts.ASK),
            Order(price('606.20'), vol('10.00000000'), ex, Consts.ASK),
            Order(price('606.51'), vol('4.84167900'), ex, Consts.ASK),
            Order(price('606.85'), vol('4.00000000'), ex, Consts.ASK),
            Order(price('606.92'), vol('1.01000000'), ex, Consts.ASK),
            Order(price('606.98'), vol('10.97000000'), ex, Consts.ASK),
        ]

        return {'bids': bids, 'asks': asks}

    def real_orderbook_2(self, price_currency='USD', vol_currency='BTC'):
        ex = self.itbit

        ex.currency = price_currency
        ex.volume_currency = vol_currency

        def price(literal):
            return Money(literal, price_currency)

        def vol(literal):
            return Money(literal, vol_currency)

        bids = [
            Order(price('605.89'), vol('58.8556'), ex, Consts.BID),
            Order(price('605.61'), vol('1.1'), ex, Consts.BID),
            Order(price('605.54'), vol('6.87'), ex, Consts.BID),
            Order(price('605.36'), vol('1.6455'), ex, Consts.BID),
            Order(price('605.09'), vol('1.413'), ex, Consts.BID),
            Order(price('604.88'), vol('1.059'), ex, Consts.BID),
            Order(price('604.7'), vol('78.7946'), ex, Consts.BID),
            Order(price('604.63'), vol('1'), ex, Consts.BID),
            Order(price('604.41'), vol('25'), ex, Consts.BID),
            Order(price('604.37'), vol('1.5642'), ex, Consts.BID),
        ]

        asks = [
            Order(price('606.62'), vol('60.8689'), ex, Consts.ASK),
            Order(price('606.85'), vol('1.5'), ex, Consts.ASK),
            Order(price('607.06'), vol('6.88'), ex, Consts.ASK),
            Order(price('607.1'), vol('1.64'), ex, Consts.ASK),
            Order(price('607.32'), vol('1.465'), ex, Consts.ASK),
            Order(price('607.59'), vol('31.83'), ex, Consts.ASK),
            Order(price('607.6'), vol('1'), ex, Consts.ASK),
            Order(price('607.82'), vol('2'), ex, Consts.ASK),
            Order(price('607.88'), vol('81.9195'), ex, Consts.ASK),
            Order(price('607.98'), vol('0.34'), ex, Consts.ASK),
        ]

        return {'bids': bids, 'asks': asks}

    def test_trivial(self):
        result = arbitrage.detect_cross(self.trivial, self.trivial)

        result.should.equal(None)

    def test_basic(self):
        """
        OB1 has a bid at 600 and OB2 has an ask at 599, both at 1btc. These should cross
        for a volume of 2btc and a profit of $1 USD.
        """

        result = arbitrage.detect_cross(self.basic_ob_2(), self.basic_ob_1())

        result.volume.should.equal(Money('1', 'BTC'))
        result.revenue.should.equal(Money('1', 'USD'))

    def test_mismatch(self):
        """
        Don't ask for cross information between two orderbooks with different volume
        currencies.
        """
        arbitrage.detect_cross.when.called_with(
            self.basic_ob_2(vol_currency='ETH'),
            self.basic_ob_1(),
        ).should.throw(
            arbitrage.MismatchedVolumeCurrenciesError,
        )

    def test_basic_non_btc(self):
        """
        OB1 has a bid at 600 and OB2 has an ask at 599, both at 1btc. These should cross
        for a volume of 2btc and a profit of $1 USD.
        """

        result = arbitrage.detect_cross(
            self.basic_ob_2(vol_currency='ETH'),
            self.basic_ob_1(vol_currency='ETH'),
        )

        result.volume.should.equal(Money('1', 'ETH'))
        result.revenue.should.equal(Money('1', 'USD'))

    def test_basic_crypto_crypto(self):
        """
        OB1 has a bid at 600 and OB2 has an ask at 599, both at 1btc. These should cross
        for a volume of 2btc and a profit of $1 USD.
        """

        result = arbitrage.detect_cross(
            self.basic_ob_2(price_currency='BTC', vol_currency='ETH'),
            self.basic_ob_1(price_currency='BTC', vol_currency='ETH'),
        )

        result.volume.should.equal(Money('1', 'ETH'))
        result.revenue.should.equal(Money('1', 'BTC'))

    def test_basic_crypto_crypto_unprofitable(self):
        """
        OB1 has a bid at 600 and OB2 has an ask at 599, both at 1btc. These should cross
        for a volume of 2btc and a profit of $1 USD.
        """

        self.itbit.market_order_fee = Decimal('0.1')
        self.bitstamp.market_order_fee = Decimal('0.1')

        result = arbitrage.detect_cross(
            self.basic_ob_2(price_currency='BTC', vol_currency='ETH'),
            self.basic_ob_1(price_currency='BTC', vol_currency='ETH'),
        )

        result.should.equal(None)

    def test_basic_crypto_crypto_minimally_profitable(self):
        """
        OB1 has a bid at 600 and OB2 has an ask at 599, both at 1btc. These should cross
        for a volume of 2btc and a profit of $1 USD.
        """
        self.itbit.market_order_fee = Decimal('0.0008')
        self.bitstamp.market_order_fee = Decimal('0.0008')

        result = arbitrage.detect_cross(
            self.basic_ob_2(price_currency='BTC', vol_currency='ETH'),
            self.basic_ob_1(price_currency='BTC', vol_currency='ETH'),
        )

        result.volume.should.equal(Money('1', 'ETH'))
        result.revenue.should.equal(Money('1', 'BTC'))
        result.fees.should.equal(Money('0.9592', 'BTC'))
        result.profit.should.equal(Money('0.0408', 'BTC'))

    def test_basic_crypto_crypto_unprofitable_with_ignore_flag_off(self):
        """
        OB1 has a bid at 600 and OB2 has an ask at 599, both at 1btc. These should cross
        for a volume of 2btc and a profit of $1 USD.
        """

        self.itbit.market_order_fee = Decimal('0.1')
        self.bitstamp.market_order_fee = Decimal('0.1')

        result = arbitrage.detect_cross(
            self.basic_ob_2(price_currency='BTC', vol_currency='ETH'),
            self.basic_ob_1(price_currency='BTC', vol_currency='ETH'),
            ignore_unprofitable=False,
        )

        result.volume.should.equal(Money('1', 'ETH'))
        result.revenue.should.equal(Money('1', 'BTC'))
        result.fees.should.equal(Money('119.9', 'BTC'))

    def test_commutative(self):
        """
        Same orderbooks as test_basic, double check it doesn't matter which order we
        give this function the orderbooks.
        """

        result = arbitrage.detect_cross(self.basic_ob_1(), self.basic_ob_2())

        result.volume.should.equal(Money('1', 'BTC'))
        result.revenue.should.equal(Money('1', 'USD'))

    def test_bigger(self):
        """
        OB1 has a bit at 600 which should cross at 599 for 0.5btc and 599.5 for 0.5 btc
        for a volume of 2btc and a profit of $0.75.
        """

        result = arbitrage.detect_cross(self.more_ob_1(), self.more_ob_2())

        result.volume.should.equal(Money('1', 'BTC'))
        result.revenue.should.equal(Money('0.75', 'USD'))

    def test_bigger_non_btc(self):
        """
        OB1 has a bit at 600 which should cross at 599 for 0.5btc and 599.5 for 0.5 btc
        for a volume of 2btc and a profit of $0.75.
        """

        result = arbitrage.detect_cross(
            self.more_ob_1(vol_currency='ETH'),
            self.more_ob_2(vol_currency='ETH'),
        )

        result.volume.should.equal(Money('1', 'ETH'))
        result.revenue.should.equal(Money('0.75', 'USD'))

    def test_bigger_crypto_crypto(self):
        """
        OB1 has a bit at 600 which should cross at 599 for 0.5btc and 599.5 for 0.5 btc
        for a volume of 2btc and a profit of $0.75.
        """

        result = arbitrage.detect_cross(
            self.more_ob_1(price_currency='BTC', vol_currency='ETH'),
            self.more_ob_2(price_currency='BTC', vol_currency='ETH'),
        )

        result.volume.should.equal(Money('1', 'ETH'))
        result.revenue.should.equal(Money('0.75', 'BTC'))

    def test_bigger_multi_fiat_price(self):
        """
        OB1 has a bit at 600 which should cross at 599 for 0.5btc and 599.5 for 0.5 btc
        for a volume of 2btc and a profit of $0.75.

        TODO:
          - in order to keep this test in the 'logic' we'll need to figure out how to
            use a mock in the forex library so we don't require an OXR account.
        """
        pass
        #result = arbitrage.detect_cross(
        #    self.more_ob_1(price_currency='USD', vol_currency='ETH'),
        #    self.more_ob_2(price_currency='CAD', vol_currency='ETH'),
        #)

        #result.volume.should.equal(Money('2', 'ETH'))
        # The revenue number will fluctuate with the exchange rate.

    def test_bigger_commutative(self):
        """
        Same as bigger, testing commutivity again.
        """

        result = arbitrage.detect_cross(self.more_ob_2(), self.more_ob_1())

        result.volume.should.equal(Money('1', 'BTC'))
        result.revenue.should.equal(Money('0.75', 'USD'))

    def test_real_data(self):
        """
        Test based on real data taken from itbit and bitstamp at about 4:30 on
        September 26 2016 against math done by hand.
        """

        result = arbitrage.detect_cross(
            self.real_orderbook_1(),
            self.real_orderbook_2(),
        )

        result.volume.should.equal(Money('12.9087178', 'BTC'))
        result.revenue.should.equal(Money('5.2679425048', 'USD'))

    def test_real_data_unprofitable(self):
        """
        Test based on real data taken from itbit and bitstamp at about 4:30 on
        September 26 2016 against math done by hand.
        """
        self.itbit.market_order_fee = Decimal('0.01')
        self.bitstamp.market_order_fee = Decimal('0.01')

        result = arbitrage.detect_cross(
            self.real_orderbook_1(),
            self.real_orderbook_2(),
        )

        result.should.equal(None)

    def test_real_data_unprofitable_flag_off(self):
        """
        Test based on real data taken from itbit and bitstamp at about 4:30 on
        September 26 2016 against math done by hand.
        """
        self.itbit.market_order_fee = Decimal('0.10')
        self.bitstamp.market_order_fee = Decimal('0.10')

        result = arbitrage.detect_cross(
            self.real_orderbook_1(),
            self.real_orderbook_2(),
            ignore_unprofitable=False,
        )

        result.volume.should.equal(Money('12.9087178', 'BTC'))
        result.revenue.should.equal(Money('5.2679425048', 'USD'))
        assert result.profit < Money('0', 'USD')

    def test_real_data_non_btc(self):
        """
        Test based on real data taken from itbit and bitstamp at about 4:30 on
        September 26 2016 against math done by hand.
        """

        result = arbitrage.detect_cross(
            self.real_orderbook_1(vol_currency='ETH'),
            self.real_orderbook_2(vol_currency='ETH'),
        )

        result.volume.should.equal(Money('12.9087178', 'ETH'))
        result.revenue.should.equal(Money('5.2679425048', 'USD'))

    def test_real_data_crypto_crypto(self):
        """
        Test based on real data taken from itbit and bitstamp at about 4:30 on
        September 26 2016 against math done by hand.
        """

        result = arbitrage.detect_cross(
            self.real_orderbook_1(price_currency='BTC', vol_currency='ETH'),
            self.real_orderbook_2(price_currency='BTC', vol_currency='ETH'),
        )

        result.volume.should.equal(Money('12.9087178', 'ETH'))
        result.revenue.should.equal(Money('5.2679425048', 'BTC'))

    def test_real_data_directional(self):
        """
        Test that the directional calculations work as expected against real data.
        """

        result1 = arbitrage.detect_directional_cross(
            self.real_orderbook_1(),
            self.real_orderbook_2(),
        )

        result1.volume.should.equal(Money('12.9087178', 'BTC'))
        result1.revenue.should.equal(Money('5.2679425048', 'USD'))

        result2 = arbitrage.detect_directional_cross(
            self.real_orderbook_2(),
            self.real_orderbook_1(),
        )

        result2.should.equal(None)

    def test_cross_falsiness(self):
        cross = arbitrage.Cross(
            Money('0', 'BTC'),
            Money('0', 'USD'),
            Money('0', 'USD'),
        )

        falsiness = False

        if cross:
            falsiness = True

        falsiness.should.equal(False)

    def test_cross_truthiness(self):
        cross = arbitrage.Cross(
            Money('1', 'BTC'),
            Money('1', 'USD'),
            Money('1', 'USD'),
        )

        falsiness = False

        if cross:
            falsiness = True

        falsiness.should.equal(True)

    def test_cross_profit(self):
        cross = arbitrage.Cross(
            Money('1', 'BTC'),
            Money('1', 'USD'),
            Money('1', 'USD'),
        )

        cross.profit.should.equal(Money('0', 'USD'))

        cross = arbitrage.Cross(
            Money('1', 'BTC'),
            Money('3.3', 'USD'),
            Money('0.03', 'USD'),
        )

        cross.profit.should.equal(Money('3.27', 'USD'))

    def test_max_buy_zero(self):
        result = arbitrage.max_buy_volume(Money('0', 'USD'), self.basic_ob_1())

        result.should.equal(Money('0', 'BTC'))

    def test_max_buy_all(self):
        result = arbitrage.max_buy_volume(Money('10000', 'USD'), self.basic_ob_1())

        result.should.equal(Money('2', 'BTC'))

    def test_max_buy_one_order(self):
        result = arbitrage.max_buy_volume(Money('599', 'USD'), self.basic_ob_1())

        result.should.equal(Money('1', 'BTC'))

    def test_max_buy_half_order(self):
        result = arbitrage.max_buy_volume(Money('299.5', 'USD'), self.basic_ob_1())

        result.should.equal(Money('0.5', 'BTC'))

    def test_max_buy_one_five_order(self):
        result = arbitrage.max_buy_volume(Money('924', 'USD'), self.basic_ob_1())

        result.should.equal(Money('1.5', 'BTC'))

    def test_max_buy_fee_simple(self):
        self.itbit.market_order_fee = Decimal('1')

        result = arbitrage.max_buy_volume(Money('599', 'USD'), self.basic_ob_1())

        result.should.equal(Money('0.5', 'BTC'))

    def test_max_buy_fee_simple(self):
        self.itbit.market_order_fee = Decimal('0.1')

        result = arbitrage.max_buy_volume(Money('599', 'USD'), self.basic_ob_1())

        result.round_to_decimal_places(3, ROUND_TRUNC).should.equal(
            Money('0.909', 'BTC'),
        )

    def test_get_executable(self):
        cross = arbitrage.detect_cross(self.basic_ob_2(), self.basic_ob_1())

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('1', 'USD'))

        buy_balance = {'USD': Money('599', 'USD'), 'BTC': Money('0', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('1', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('1', 'BTC'))

    def test_get_executable_no_buy_balance(self):
        cross = arbitrage.detect_cross(self.basic_ob_2(), self.basic_ob_1())

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('1', 'USD'))

        buy_balance = {'USD': Money('0', 'USD'), 'BTC': Money('0', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('1', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('0', 'BTC'))

    def test_get_executable_no_buy_balance_b(self):
        cross = arbitrage.detect_cross(self.basic_ob_2(), self.basic_ob_1())

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('1', 'USD'))

        buy_balance = {'USD': Money('0', 'USD'), 'BTC': Money('0', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('1000', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('0', 'BTC'))

    def test_get_executable_no_sell_balance(self):
        cross = arbitrage.detect_cross(self.basic_ob_2(), self.basic_ob_1())

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('1', 'USD'))

        buy_balance = {'USD': Money('599', 'USD'), 'BTC': Money('0', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('0', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('0', 'BTC'))

    def test_get_executable_no_sell_balance_b(self):
        cross = arbitrage.detect_cross(self.basic_ob_2(), self.basic_ob_1())

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('1', 'USD'))

        buy_balance = {'USD': Money('599', 'USD'), 'BTC': Money('1000000', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('0', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('0', 'BTC'))

    def test_get_executable_crypto_crypto(self):
        cross = arbitrage.detect_cross(
            self.basic_ob_2(price_currency='BTC', vol_currency='ETH'),
            self.basic_ob_1(price_currency='BTC', vol_currency='ETH'),
        )

        cross.volume.should.equal(Money('1', 'ETH'))
        cross.revenue.should.equal(Money('1', 'BTC'))

        buy_balance = {'BTC': Money('599', 'BTC'), 'ETH': Money('1000000', 'ETH')}
        sell_balance = {'BTC': Money('0', 'BTC'), 'ETH': Money('1', 'ETH')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('1', 'ETH'))

    def test_get_executable_half(self):
        cross = arbitrage.detect_cross(
            self.basic_ob_2(price_currency='BTC', vol_currency='ETH'),
            self.basic_ob_1(price_currency='BTC', vol_currency='ETH'),
        )

        cross.volume.should.equal(Money('1', 'ETH'))
        cross.revenue.should.equal(Money('1', 'BTC'))

        buy_balance = {'BTC': Money('299.5', 'BTC'), 'ETH': Money('1000000', 'ETH')}
        sell_balance = {'BTC': Money('0', 'BTC'), 'ETH': Money('1', 'ETH')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('0.5', 'ETH'))

    def test_get_executable_half(self):
        cross = arbitrage.detect_cross(
            self.basic_ob_2(price_currency='BTC', vol_currency='ETH'),
            self.basic_ob_1(price_currency='BTC', vol_currency='ETH'),
        )

        cross.volume.should.equal(Money('1', 'ETH'))
        cross.revenue.should.equal(Money('1', 'BTC'))

        buy_balance = {'BTC': Money('299.5', 'BTC'), 'ETH': Money('1000000', 'ETH')}
        sell_balance = {'BTC': Money('0', 'BTC'), 'ETH': Money('1', 'ETH')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('0.5', 'ETH'))

    def test_get_executable_restriction(self):
        cross = arbitrage.detect_cross(self.more_ob_1(), self.more_ob_2())

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('0.75', 'USD'))

        buy_balance = {'USD': Money('299.5', 'USD'), 'BTC': Money('0', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('1', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('0.5', 'BTC'))

    def test_get_executable_full_second_order(self):
        cross = arbitrage.detect_cross(self.more_ob_1(), self.more_ob_2())

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('0.75', 'USD'))

        buy_balance = {'USD': Money('599.25', 'USD'), 'BTC': Money('0', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('1', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('1', 'BTC'))

    def test_get_executable_partial_second_order(self):
        cross = arbitrage.detect_cross(self.more_ob_1(), self.more_ob_2())

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('0.75', 'USD'))

        buy_balance = {'USD': Money('449.375', 'USD'), 'BTC': Money('0', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('1', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('0.75', 'BTC'))

    def test_get_executable_simple_fee(self):
        self.itbit.market_order_fee = 1
        self.bitstamp.market_order_fee = 1

        cross = arbitrage.detect_cross(
            self.basic_ob_2(),
            self.basic_ob_1(),
            ignore_unprofitable=False,
        )

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('1', 'USD'))

        buy_balance = {'USD': Money('599', 'USD'), 'BTC': Money('0', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('1', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.should.equal(Money('0.5', 'BTC'))

    def get_test_executable_realistic_fee(self):
        self.itbit.market_order_fee = Decimal('0.0001')
        self.bitstamp.market_order_fee = Decimal('0.0001')

        cross = arbitrage.detect_cross(
            self.basic_ob_2(),
            self.basic_ob_1(),
            ignore_unprofitable=False,
        )

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('1', 'USD'))

        buy_balance = {'USD': Money('599', 'USD'), 'BTC': Money('0', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('1', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.round_to_decimal_places(8).should.equal(Money('0.99990001', 'BTC'))

    def test_get_executable_fee_complex(self):
        self.itbit.market_order_fee = Decimal('0.0001')
        self.bitstamp.market_order_fee = Decimal('0.0001')

        cross = arbitrage.detect_cross(self.more_ob_1(), self.more_ob_2())

        cross.volume.should.equal(Money('1', 'BTC'))
        cross.revenue.should.equal(Money('0.75', 'USD'))

        buy_balance = {'USD': Money('599.25', 'USD'), 'BTC': Money('0', 'BTC')}
        sell_balance = {'USD': Money('0', 'USD'), 'BTC': Money('1', 'BTC')}

        result = arbitrage.get_executable_volume(cross, buy_balance, sell_balance)

        result.round_to_decimal_places(8, ROUND_TRUNC)\
            .should.equal(Money('0.99990005', 'BTC'))

    def test_get_many_simple(self):
        orderbooks = [self.basic_ob_2(), self.basic_ob_1()]

        crosses = arbitrage.detect_crosses_between_many_orderbooks(orderbooks)

        len(crosses).should.equal(1)

        crosses[0].volume.should.equal(Money('1', 'BTC'))
        crosses[0].revenue.should.equal(Money('1', 'USD'))

    def test_get_many_fees(self):
        self.bitstamp.market_order_fee = Decimal('0.01')

        orderbooks = [self.basic_ob_2(), self.basic_ob_1()]

        crosses = arbitrage.detect_crosses_between_many_orderbooks(orderbooks)

        len(crosses).should.equal(0)

    def test_get_many_fees_ignore_false(self):
        self.bitstamp.market_order_fee = Decimal('0.001')

        orderbooks = [self.basic_ob_2(), self.basic_ob_1()]

        crosses = arbitrage.detect_crosses_between_many_orderbooks(
            orderbooks,
            ignore_unprofitable=False,
        )

        len(crosses).should.equal(1)

    def test_get_many_two_crosses(self):
        orderbooks = [self.basic_ob_2(), self.basic_ob_2(), self.basic_ob_1()]

        crosses = arbitrage.detect_crosses_between_many_orderbooks(orderbooks)

        len(crosses).should.equal(2)
        crosses[0].volume.should.equal(Money('1', 'BTC'))
        crosses[0].revenue.should.equal(Money('1', 'USD'))
        crosses[1].volume.should.equal(Money('1', 'BTC'))
        crosses[1].revenue.should.equal(Money('1', 'USD'))

    def test_get_many_two_separate_crosses(self):
        orderbooks = [
            self.basic_ob_2(),
            self.basic_ob_1(),
            self.more_ob_2(),
            self.more_ob_1(),
        ]

        crosses = arbitrage.detect_crosses_between_many_orderbooks(orderbooks)

        len(crosses).should.equal(4)
        crosses[0].revenue.should.equal(Money('1', 'USD'))
        crosses[0].volume.should.equal(Money('1', 'BTC'))
        crosses[-1].volume.should.equal(Money('1', 'BTC'))
        crosses[-1].revenue.should.equal(Money('0.75', 'USD'))




import prompter
import termcolor as tc

from gryphon.lib.exchange import exchange_factory
from gryphon.lib.logger import get_logger
from gryphon.lib.models.trade import Trade
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money
from gryphon.lib import session

logger = get_logger(__name__)

def buyback():
    prompt_msg = tc.colored('Did you stop the Coinbase Bot before running this?', 'red')
    bot_stopped = prompter.yesno(prompt_msg)
    if not bot_stopped:
        print tc.colored('Go stop the bot first.', 'red')
        return

    db = session.get_a_trading_db_mysql_session()
    try:
        trades_with_outstanding_fees = db.query(Trade)\
            .filter(Trade.has_outstanding_btc_fee)\
            .all()

        transactions_with_outstanding_fees = db.query(Transaction)\
            .filter(Transaction.has_outstanding_btc_fee)\
            .all()

        coinbase_exchange_data = exchange_factory.make_exchange_data_from_key('COINBASE', db)

        trades_buyback_amount = sum([t.fee for t in trades_with_outstanding_fees])
        transactions_buyback_amount = sum([t.fee for t in transactions_with_outstanding_fees])
        btc_buyback_amount = trades_buyback_amount + transactions_buyback_amount

        print 'Go buy %s on Coinbase (not the exchange)' % btc_buyback_amount

        prompt_msg = 'How much USD did it cost (total including Coinbase Fee): USD'
        raw_usd_cost = prompter.prompt(prompt_msg)
        usd_cost = Money(raw_usd_cost, 'USD')

        withdrawal = Transaction(
            Transaction.WITHDRAWL,
            Transaction.IN_TRANSIT,
            usd_cost,
            coinbase_exchange_data,
            {'notes': 'BTC Fee Buyback'},
        )
        deposit = Transaction(
            Transaction.DEPOSIT,
            Transaction.IN_TRANSIT,
            btc_buyback_amount,
            coinbase_exchange_data,
            {'notes': 'BTC Fee Buyback'},
        )

        withdrawal.complete()
        deposit.complete()

        for trade in trades_with_outstanding_fees:
            trade.fee_buyback_transaction = deposit

        for transaction in transactions_with_outstanding_fees:
            transaction.fee_buyback_transaction = deposit

    finally:
        session.commit_mysql_session(db)
        db.remove()



from gryphon.data_service.pollers.orderbook.orderbook_poller import OrderbookPoller
from gryphon.lib.exchange.kraken_btc_eur import KrakenBTCEURExchange


class KrakenOrderbook(OrderbookPoller):
    def __init__(self):
        self.exchange_name = u'KRAKEN_BTC_EUR'
        self.currency = 'EUR'
        self.poll_time = 2
        # How many orders to get. Kraken appears to have a limit of 500 right now,
        # but this number is future-proofed.
        self.orderbook_depth = 100000

    @property
    def pair(self):
        return KrakenBTCEURExchange.construct_pair(self.currency)

    @property
    def url(self):
        return 'https://api.kraken.com/0/public/Depth?pair=%s&count=%s' % (
            self.pair,
            self.orderbook_depth,
        )

    # API response format:
    # {"result":{"XXBTZEUR":{"bids":[["202.33000","6.658",1442880257], ...

    def get_bids_and_asks(self, response):
        orderbook = response['result'][self.pair]
        raw_bids = orderbook['bids']
        raw_asks = orderbook['asks']
        return raw_bids, raw_asks

    def parse_order(self, order):
        return [order[0], order[1], order[2]]



"""
A simple library for common tasks in market making strategies.
"""

from gryphon.lib.metrics import midpoint as midpoint_lib


def midpoint_centered_fixed_spread(orderbook, spread, quote_depth=None):
    """
    """
    midpoint = midpoint_lib.get_midpoint_from_orderbook(orderbook, depth=quote_depth)

    bid_price = midpoint - (midpoint * spread)
    ask_price = midpoint + (midpoint * spread)

    return bid_price, ask_price


def simple_position_responsive_sizing(max_position, current_position):
    """
    Simple order sizing logic for a market making strategy. Define a 'max_position' that
    our strategy is will to take, and place orders of that size on both sides of the
    orderbook, less whatever position we currently are in.

    e.g. position: -0.5 BTC, max_position: 2 BTC:
        return 2 BTC, 1.5 BTC

    e.g. position: 2 BTC, max_position, 2 BTC:
        return 0 BTC, 2 BTC
    """
    bid_volume = max_position
    ask_volume = max_position

    if current_position > 0:
        bid_volume = bid_volume - min(current_position, max_position)

    if current_position < 0:
        ask_volume = ask_volume - min((-1) * current_position, max_position)

    return bid_volume, ask_volume




import hashlib

from cdecimal import *
from delorean import Delorean

from gryphon.execution.lib import conf
from gryphon.lib.exchange.exchange_factory import make_exchange_from_key
from gryphon.lib.logger import get_logger
from gryphon.lib.money import Money
from gryphon.lib.models.exchange import Exchange
from gryphon.lib.models.exchange import Transaction
from gryphon.lib.session import commit_mysql_session
from gryphon.lib.util.profile import tick_profile

logger = get_logger(__name__)

TRANSFER_UNIT = Money('30', 'BTC')


# this gives us a unique amount to add to withdrawal transactions
# they will look like 10.00xxxx00 where xxxx is the unique number
# we can't use the last 2 decimals because of OkCoin's balance rounding
def create_uniquifier():
    hasher = hashlib.sha1()
    hasher.update(str(Delorean().epoch))
    uniquifier = int(hasher.hexdigest(), 16) % 10000
    uniquifier *= Decimal("0.000001")
    assert(uniquifier < Decimal("0.01"))
    return uniquifier


def get_deposit_enabled_exchanges(db):
    exchange_targets = db.query(Exchange)\
        .filter(Exchange.name.in_(conf.deposits_enabled))\
        .all()

    return exchange_targets


def get_destination_exchange(algo, exchange_data, exchanges):
    full_balances = {}

    for exchange in exchanges:
        full_balances[exchange.name] = exchange.full_balance
        
    bitcoin_total = sum([full_balances[e.name]['BTC'] for e in exchanges])

    rules = {}

    # target == proportion of total bitcoins and max is TRANSFER_UNIT + target
    for exchange in exchanges:
        target = conf.proportions[exchange.name] * bitcoin_total
        maximum = TRANSFER_UNIT + target
        rules[exchange.name] = {'target': target, 'max': maximum}

        algo.log('%s - Bal:%s - w/inc:%s - Target:%s - Max BTC:%s', (exchange.name, exchange.balance['BTC'], full_balances[exchange.name]['BTC'], target, maximum ), 'yellow', log_level='debug')
   
    if exchange_data.balance['BTC'] < rules[exchange_data.name]['max']:
        algo.log('%s is not above its maximum (%s), not sending BTC', (exchange_data.name, rules[exchange_data.name]['max']), 'yellow')

        return
   
    # remove all exchanges with more than their target
    exchanges = [e for e in exchanges if full_balances[e.name]['BTC'] < rules[e.name]['target']]

    # remove exchanges with more than their maxiumum allowed total capital
    for e in exchanges:
        if full_balances[e.name].total_usd_value() >= conf.account_value_limits[e.name]:
            algo.log("Filtering out %s due to account value (%s) above limit (%s)", (e.name, full_balances[e.name].total_usd_value(), conf.account_value_limits[e.name]), 'yellow', log_level='debug')

    exchanges = [e for e in exchanges if full_balances[e.name].total_usd_value() < conf.account_value_limits[e.name]]
    
    # remove exchanges that our current bot isn't allowed to send to
    exchanges = [e for e in exchanges if e.name not in conf.forbidden_targets[exchange_data.name]]

    if not exchanges:
        algo.log('No Exchanges to Send BTC', (), 'yellow')
        return
    
    # sort exchanges into order by how close they are to their target
    exchanges.sort(key=lambda e: (full_balances[e.name]['BTC'] / rules[e.name]['target']))
    
    destination_exchange_data = exchanges[0] 

    return destination_exchange_data


def send_btc_from_exchange_to_exchange(self, source_exchange, destination_exchange, amount):
    raise Exception("""The code in the gryphon framework for transferring crypto
        around does not have a running test suite and should be used with caution. If
        you wish to use these features, simply remove this warning from your own copy
        of the codebase.""")

    destination_address = destination_exchange.current_deposit_address

    logger.info('Sending %s from %s to %s (%s)' % (
        amount,
        source_exchange.name,
        destination_exchange.name,
        destination_address,
    ))

    slacker = Slacker('#balancer','balancer', icon_emoji=':zap:')
    slacker.notify('Sending %s from %s to %s (%s)' % (
        amount,
        source_exchange.name,
        destination_exchange.name,
        destination_address,
    ))

    result = source_exchange.withdraw_crypto(destination_address, amount)

    return (
        destination_address,
        result.get('tx'),
        result.get('exchange_withdrawal_id'),
    )

@tick_profile
def bitcoin_rebalance(db, exchange_data, algo, execute=False):
    algo.log('------- Bitcoin Balancer Report -----------', (), 'yellow')

    if exchange_data.name not in conf.withdrawals_enabled:
        algo.log('Withdrawals not enabled for %s, not balancing', exchange_data.name, 'yellow')
        return
    
    exchanges = get_deposit_enabled_exchanges(db)

    destination_exchange_data = get_destination_exchange(algo, exchange_data, exchanges)

    if destination_exchange_data is None:
        return
    elif destination_exchange_data == exchange_data:
        algo.log('%s not sending to self', exchange_data.name, 'yellow')
        return
    
    source_exchange = make_exchange_from_key(exchange_data.name)
    destination_exchange = make_exchange_from_key(destination_exchange_data.name)

    transfer_amount = TRANSFER_UNIT + create_uniquifier()

    algo.log('Sending %s from %s to %s', (transfer_amount, source_exchange.name, destination_exchange.name), 'yellow')

    if execute:
        # Send the bitcoins and record the transactions.

        deposit_address, transaction_hash, exchange_withdrawal_id = send_btc_to_exchange(source_exchange, destination_exchange, transfer_amount)

        transactions = exchange_data.record_withdrawal(
            destination_exchange_data,
            transfer_amount,
            deposit_address,
            transaction_hash,
            exchange_withdrawal_id,
        )

        commit_mysql_session(db)
    else:
        algo.log('Not Sending Because of NO EXECUTE', (), 'yellow')




# -*- coding: utf-8 -*-
import os

from alchimia import TWISTED_STRATEGY
from sqlalchemy import create_engine
from twisted.internet import reactor


class Auditor(object):
    def start(self):
        raise NotImplementedError

    def audit(self):
        raise NotImplementedError

    def setup_mysql(self):
        self.engine = create_engine(
            os.environ.get('DB_CRED'),
            reactor=reactor,
            strategy=TWISTED_STRATEGY,
        )




"""
This is a simple market making strategy to demonstrate use of the Gryphon
framework. It follows the same tick-logic as SuperSimpleMarketMaking, but it's target
exchange, spread, and base volume, are all configurable.
"""

from cdecimal import Decimal

from gryphon.execution.strategies.base import Strategy
from gryphon.lib import market_making as mm
from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts


class SimpleMarketMaking(Strategy):
    def __init__(self, db, harness, strategy_configuration):
        super(SimpleMarketMaking, self).__init__(db, harness)

        # Configurable properties with defaults.
        self.spread = Decimal('0.01')
        self.base_volume = Money('0.005', 'BTC')
        self.exchange = None

        self.configure(strategy_configuration)

    def configure(self, strategy_configuration):
        super(SimpleMarketMaking, self).configure(strategy_configuration)

        self.init_configurable('spread', strategy_configuration)
        self.init_configurable('base_volume', strategy_configuration)
        self.init_configurable('exchange', strategy_configuration)
        self.init_primary_exchange()

    def init_primary_exchange(self):
        self.primary_exchange = self.harness.exchange_from_key(self.exchange)

        # This causes us to always audit our primary exchange.
        self.target_exchanges = [self.primary_exchange.name]

    def tick(self, current_orders):
        self.primary_exchange.cancel_all_open_orders()

        ob = self.primary_exchange.get_orderbook()

        bid_price, ask_price = mm.midpoint_centered_fixed_spread(ob, self.spread)

        bid_volume, ask_volume = mm.simple_position_responsive_sizing(
            self.base_volume,
            self.position,
        )

        if bid_volume > 0:
            self.primary_exchange.limit_order(Consts.BID, bid_volume, bid_price)

        if ask_volume > 0:
            self.primary_exchange.limit_order(Consts.ASK, ask_volume, ask_price)




"""Initial Migration

Revision ID: 5169cd803822
Revises: None
Create Date: 2019-03-18 15:12:05.563812

"""

# revision identifiers, used by Alembic.
revision = '5169cd803822'
down_revision = None

from alembic import op
import sqlalchemy as sa

from gryphon.dashboards.models.columns.password_column import PasswordColumn
sa.PasswordColumn = PasswordColumn

def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('user',
    sa.Column('user_id', sa.Integer(), nullable=False),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('username', sa.Unicode(length=64), nullable=False),
    sa.Column('password', sa.PasswordColumn(length=100), nullable=False),
    sa.Column('user_type', sa.Unicode(length=64), nullable=False),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.PrimaryKeyConstraint('user_id')
    )
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('user')
    ### end Alembic commands ###



import pyximport; pyximport.install()

from gryphon.lib.exchange.kraken_btc_eur import KrakenBTCEURExchange
from gryphon.tests.exceptional.exchange_wrappers.auth_methods import ExchangeAuthMethodsTests


class TestKrakenBTCEURAuthMethods(ExchangeAuthMethodsTests):
    def setUp(self):
        self.exchange = KrakenBTCEURExchange()



# -*- coding: utf-8 -*-
import json

from delorean import Delorean
import treq
from twisted.internet import defer
from twisted.internet.task import LoopingCall
from twisted.python import log

from gryphon.data_service.pollers.poller import Poller
import gryphon.data_service.util as util


class RequestPoller(Poller):
    @defer.inlineCallbacks
    def start(self):
        self.redis = yield util.setup_redis()
        self.looping_call = LoopingCall(self.get_request)
        self.looping_call.start(self.poll_time)

    def get_request(self):
        agent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:32.0) Gecko/20100101 Firefox/32.0"
        d = treq.get(self.url, headers={'user-agent': agent})
        d.addCallback(self.get_response)
        d.addCallback(self.heartbeat)
        d.addErrback(self.log_request_error)

        return d

    def log_request_error(self, error):
        log.err('Error in Request: %s' % str(error))

    def get_response(self, response):
        # Parse the Response and check for errors
        d = treq.content(response)

        # We want to add parse_float=Decimal, but it currently breaks json writing
        # in later code paths
        d.addCallback(json.loads)
        d.addCallback(self.parse_response)
        d.addErrback(self.log_response_error, response)

        return d

    @defer.inlineCallbacks
    def heartbeat(self, response=None):
        yield self.redis.set(self.heartbeat_key, Delorean().epoch)

    def log_response(self, resp_text, label='', *args, **kwargs):
        log.msg(u'%s:%s' % (label, resp_text[:100]))

    def log_response_error(self, error_text, response, *args, **kwargs):
        treq.text_content(response).addCallback(
            self.log_response,
            u'Error in Response from URL:%s %s' % (self.url, error_text),
            log.err,
        )



# Simple test script for benchmarking regular python logging vs twisted's logging.

import logging

from twisted.internet import reactor

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

log_counter = 0


def std_log():
    global log_counter
    logger.info("STD LOG TEST")
    log_counter += 1
    reactor.callLater(0, std_log)


def tx_log():
    global log_counter
    logger.info("TX  LOG TEST")
    log_counter += 1
    reactor.callLater(0, tx_log)


def stop():
    print "Log Counter: %s" % log_counter
    reactor.stop()


reactor.callLater(0, std_log)
reactor.callLater(10, stop)

reactor.run()

# Results 2015-09-24
# std_log, 10s: 88,221 logs
# tx_log, 10s: 87,307 logs
# Conclusion: negligible difference



import pyximport; pyximport.install()

from gryphon.lib.exchange.quadriga_btc_cad import QuadrigaBTCCADExchange
from gryphon.tests.exceptional.exchange_wrappers.auth_methods import ExchangeAuthMethodsTests


class TestQuadrigaBTCCADAuthMethods(ExchangeAuthMethodsTests):
    def setUp(self):
        self.exchange = QuadrigaBTCCADExchange()



# -*- coding: utf-8 -*-
from cdecimal import *
from datetime import datetime, date, timedelta
import json
import os
import uuid

from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, Numeric
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship, backref

from gryphon.execution.models.backtesting.result import Result
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.models.base import Base
from gryphon.lib.money import Money

metadata = Base.metadata


class ResultTrade(Base):
    __tablename__ = 'result_trade'
    
    trade_id = Column(Integer, primary_key=True)
    
    _price = Column('price', Numeric(precision=20, scale=10))
    _price_currency = Column('price_currency', Unicode(3))
    _volume = Column('volume', Numeric(precision=20, scale=10))
    _volume_currency = Column('volume_currency', Unicode(3))
    tick = Column(Integer)
    
    user_trade = Column(Unicode(64))
    result_id = Column(Integer, ForeignKey('result.result_id'))
    
    def __init__(self, trade_obj):
        self.price = trade_obj.price
        self.volume = trade_obj.volume
        self.tick = trade_obj.tick
        if trade_obj.bid.order_id:
            self.user_trade = Consts.BID
        elif trade_obj.ask.order_id:
            self.user_trade = Consts.ASK
        else:
            self.user_trade = u'NO'

    @property
    def volume(self):
        return Money(self._volume, self._volume_currency)

    @volume.setter
    def volume(self, value):
        self._volume = value.amount
        self._volume_currency = value.currency

    @property
    def price(self):
        return Money(self._price, self._price_currency)

    @price.setter
    def price(self, value):
        self._price = value.amount
        self._price_currency = value.currency



# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.trades.trades_poller import TradesPoller
from gryphon.lib.exchange.kraken_btc_eur import KrakenBTCEURExchange
from gryphon.lib.money import Money


class KrakenTrades(TradesPoller):
    def __init__(self):
        self.exchange_name = u'KRAKEN_BTC_EUR'
        self.currency = 'EUR'
        self.poll_time = 10

    @property
    def pair(self):
        return KrakenBTCEURExchange.construct_pair(self.currency)

    @property
    def url(self):
        return 'https://api.kraken.com/0/public/Trades?pair=%s' % self.pair

    def parse_trades(self, resp_obj):
        trades = []

        for trade in resp_obj['result'][self.pair]:
            trade = {
                'exchange': self.exchange_name,
                'price': Money(trade[0], self.currency),
                'volume': Money(trade[1], 'BTC'),
                'timestamp': trade[2],
                'trade_id': int(trade[2] * 10000),
            }

            trades.append(trade)

        return trades



# -*- coding: utf-8 -*-
import os
from base import Base
import json
import uuid
from datetime import datetime
from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, BigInteger, Numeric

metadata = Base.metadata

class OrderbookSnapshot(Base):
    __tablename__ = 'orderbook_snapshot'
   
    unique_id = Column(Unicode(64), nullable=False)
    orderbook_id = Column(Integer, primary_key=True)
    data = Column(UnicodeText(length=2**31))
    time_retrieved = Column(DateTime, nullable=False)
 
    def __init__(self, data, time_retrieved):
        self.unique_id = u'ord_%s' % unicode(uuid.uuid4().hex)
        self.time_retrieved = time_retrieved
        self.data = data
        
    def __unicode__(self):
        return u'Orderbook_Snapshot'
        
    def __repr__(self):
        return json.dumps({
            'data': self.data,
            'time_retrieved': unicode(self.time_retrieved),
        }, ensure_ascii=False)



def standard_deviation(tf,prices):
    sd= []
    sddate = []
    x = tf
    while x <= len(prices):
        array2consider = prices[x-tf:x]
        standev = array2consider.std()
        sd.append(standev)
        sddate.append(date[x])
        x+=1
    return sddate,sd



"""
A simple script that allows us to run our test-suite from an installed console entry
point.
"""

import pyximport; pyximport.install()

import logging
import os
from pkg_resources import resource_filename
import sys

import nose

import money

def main():
    test_dir = resource_filename('gryphon', 'tests/logic')

    if 'extra' in sys.argv:
        test_dir = resource_filename('gryphon', 'tests/extra')
    elif 'environment' in sys.argv:
        test_dir = resource_filename('gryphon', 'tests/environment')

    args = [
        '-s',
        '--rednose',
        '--where=%s' % test_dir,
    ]

    nose.run(argv=args)



import numpy as np
import time

sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData,
                                                 delimiter=',',
                                                 unpack=True)


def cmo(prices,tf):

    CMO = []
    x = tf
    while x < len(date):
        considerationPrices = prices[x-tf:x]
        #print considerationPrices

        upSum = 0
        downSum = 0

        y = 1
        
        while y < tf:
            curPrice = considerationPrices[y]
            prevPrice = considerationPrices[y-1]

            if curPrice >= prevPrice:
                upSum+= (curPrice-prevPrice)

            else:
                downSum += (prevPrice-curPrice)

            y+=1

        #print upSum
        #print downSum
        curCMO = ((upSum-downSum)/(upSum+float(downSum)))*100.00
        #print curCMO
        CMO.append(curCMO)
        
        x+=1

    return date[tf:],CMO

cmo(closep,10)



# -*- coding: utf-8 -*-
# This creates the declarative base for the analytics database models, which must be
# a different declarative base than for any of our other database, because they may
# share some table names.
# Reference: http://stackoverflow.com/questions/8264686/sqlalchemy-multiple-databases-with-same-table-names-not-working

from sqlalchemy.ext.declarative import declarative_base

AtlasZeroBase = declarative_base()
metadata = AtlasZeroBase.metadata


def unicode_string(self):
    return unicode(self).encode('utf-8')

AtlasZeroBase.__str__ == unicode_string


# How to migrate a database

#   foreman run alembic revision --autogenerate -m "moved value to Text from String"
# This generates the change script

#   foreman run alembic upgrade head
# This Executes the latest change script.



from decimal import Decimal
from collections import defaultdict
def price_diff(fundamental_values, timestamps, window_time=300):
    assert len(timestamps) == len(fundamental_values)
    
    fundamental_values = list(fundamental_values)
    fundamental_values.reverse()
    timestamps = list(timestamps)
    timestamps.reverse()
    
    price_diffs = []
    for i in range(len(timestamps)):
        curr_fv = fundamental_values[i]
        curr_ts = timestamps[i]
        lookback_index = 0
        for j in range(len(timestamps)):
            index = i + j
            if index > len(timestamps)-1:
                break
            if curr_ts - timestamps[index] > window_time:
                lookback_index = index
                break
        if lookback_index == 0:
            price_diff = Decimal('0')
        else:
            price_diff = (abs(fundamental_values[lookback_index] - curr_fv) / fundamental_values[lookback_index])
        price_diffs.append(price_diff)
    price_diffs.reverse()
    return price_diffs

def price_diff_rolling_average(fund_vals, fund_val_timestamps, window_time=60, rolling_period=86400):
    price_diffs = price_diff(fund_vals, fund_val_timestamps, window_time)
    
    price_diffs = list(price_diffs)
    price_diffs.reverse()
    timestamps = list(fund_val_timestamps)
    timestamps.reverse()
    
    price_diff_averages = []
    matched_timestamps = []
    for i in range(len(timestamps)):
        current_timestamp = timestamps[i]
        
        lookback_index= 0
        for j in range((len(fund_val_timestamps))):
            index = i + j
            if index > len(timestamps) -1:
                break
            if current_timestamp - timestamps[index] > rolling_period:
                lookback_index = index
                break
        if lookback_index != 0:
            ts = timestamps[i]
            relevant_list = price_diffs[i:lookback_index]
            if relevant_list:
                a = sum(relevant_list) / len(relevant_list)
            else:
                a = Decimal('0')
            price_diff_averages.append(a)
        matched_timestamps.append(current_timestamp)
    price_diff_averages.reverse()
    return matched_timestamps, price_diff_averages


def price_diff_compare(fund_vals, fund_val_timestamps, alt_fund_vals, alt_fund_val_timestamps, window_time=50):
    assert len(fund_vals) == len(fund_val_timestamps) and len(alt_fund_vals) == len(alt_fund_val_timestamps)
    price_diffs = price_diff(fund_vals, fund_val_timestamps, window_time)
    alt_price_diffs = price_diff(alt_fund_vals, alt_fund_val_timestamps, window_time)
    timeline = defaultdict(dict)
    for i in range(len(fund_val_timestamps)):
        ts = fund_val_timestamps[i]
        timeline[ts]['fund']=price_diffs[i]
    
    current_price_diff = price_diffs[0]
    for i in range(len(alt_fund_val_timestamps)):
        ts = alt_fund_val_timestamps[i]
        timeline[ts]['alt'] = alt_price_diffs[i]
    
    diffs = {}
    current_alt = 0
    current_fund = 0
    for ts, val in timeline.iteritems():
        if 'alt' in val:
            current_alt = val['alt']
        if 'fund' in val:
            current_fund = val['fund']
        
        diffs[ts] = abs(current_alt - current_fund)
            #timetstamps, diffs
    sorted_timestamps = sorted(diffs, key=lambda key: key)  
    return sorted_timestamps, [diffs[ts] for ts in sorted_timestamps]



"""
Test an exchange's authenticated endpoints that don't make any modifying calls.
"""

import pyximport; pyximport.install()
import gryphon.lib; gryphon.lib.prepare()

import logging
import os
import unittest
import sure
import mock

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money


class ExchangeAuthMethodsTests(object):
    def test_balance(self):
        balance = self.exchange.get_balance()

        assert self.exchange.currency in balance 
        assert self.exchange.volume_currency in balance

    def test_open_orders(self):
        open_orders = self.exchange.get_open_orders()

    def test_recent_trades(self):
        if hasattr(self.exchange, '_get_recent_trades'):
            recent_trades = self.exchange._get_recent_trades()

    def test_get_audit_data(self):
        audit_data = self.exchange.get_order_audit_data()

    def test_price_quote(self):
        quote = self.exchange.get_price_quote(Consts.BID, Money('1', 'BTC'))

    def test_deposit_address(self):
        deposit_address = self.exchange.current_deposit_address



from tinkerpy.models.trade import Trade
from tinkerpy.models.order import Order

import sqlalchemy
from sqlalchemy.orm import joinedload

from delorean import Delorean
from decimal import Decimal


""" 
    Old versions that were used up until about November 2014 when multi
    became our main algorithm. We may still want these in the future so
    I'm leaving them in a new file.
"""

def old_get_multi_trades_for_period(self, start_time, end_time):
    # this gets the union of all multi trades and cavirtex/multi trades

    # cavirtex/vault trades that are not multi trades
    multi_trades = self.trading_db.query(Trade)\
        .join(Order)\
        .filter(Order._exchange_name.in_((
            "CAVIRTEX", 
            "VAULTOFSATOSHI"))
        )\
        .filter(~Order.actor.in_(["Manual", "Multi", "runner[nightly_position_close]"]))\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .options(joinedload('order'))\
        .all()

    # all multi trades
    multi_trades += self.trading_db.query(Trade)\
        .join(Order)\
        .filter(Order.actor == "Multi")\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .options(joinedload('order'))\
        .all()

    return multi_trades


def old_get_linear_trades_for_period(self, start_time, end_time):
    exchanges = ['BITSTAMP', 'BITFINEX', 'KRAKEN', 'ITBIT']

    trades = self.trading_db.query(Trade)\
        .join(Order)\
        .filter(Order._exchange_name.in_(exchanges))\
        .filter(~Order.actor.in_(["Manual", "Multi", "runner[nightly_position_close]"]))\
        .filter(Trade.time_created >= start_time)\
        .filter(Trade.time_created < end_time)\
        .options(joinedload('order'))\
        .all()

    return trades





import pyximport; pyximport.install()

from gryphon.lib.exchange.kraken_btc_eur import KrakenBTCEURExchange
from gryphon.tests.logic.exchange_wrappers.public_methods import ExchangePublicMethodsTests


class TestKrakenBTCEURPublicMethods(ExchangePublicMethodsTests):
    def setUp(self):
        self.exchange = KrakenBTCEURExchange()




"""
A decorator that retries exchange requests if they fail.
"""

import functools
from retrying import Retrying

import gryphon.lib.exchange.exceptions as exceptions


def retry_if_exchange_exception(exception):
    return isinstance(exception, exceptions.ExchangeAPIFailureException) or isinstance(exception, exceptions.NonceError)


def exchange_retry():
    """
    Decorator that retries exchange requests with an exponential backoff between 1 and 128
    seconds.
    """
    def decorator(function):
        @functools.wraps(function)
        def wrapper(*args, **kwargs):
            return Retrying(
                retry_on_exception=retry_if_exchange_exception,
                wait_exponential_multiplier=1000,
                stop_max_attempt_number=7,
            ).call(function, *args, **kwargs)
        return wrapper
    return decorator





import os
import logging
import logging.handlers


def get_logger(name, debug=False):
    logging.basicConfig(level=logging.INFO)
    logger = logging.getLogger(name)
    if debug:
        logger.setLevel(logging.DEBUG)
    if 'SYSLOG_ADDRESS' in os.environ:
        syslog = logging.handlers.SysLogHandler(address=(os.environ['SYSLOG_ADDRESS'], int(os.environ['SYSLOG_PORT'])))
        formatter = logging.Formatter('%(name)s: %(levelname)s %(message)s')
        syslog.setFormatter(formatter)
        logger.addHandler(syslog)
    return logger



import termcolor as tc

from gryphon.execution.lib.exchange_color import exchange_color, legend
from gryphon.lib.exchange.exchange_factory import *
from gryphon.lib.exchange.exchange_order import Order as ExchangeOrder
from gryphon.lib.exchange.exceptions import ExchangeAPIErrorException, ExchangeAPIFailureException
from gryphon.lib.logger import get_logger

logger = get_logger(__name__)


def start_order_book_requests(exchanges):
    order_book_requests = []

    for exchange in exchanges:
        order_book_requests.append(exchange.get_orderbook_req())

    return order_book_requests


def start_open_order_requests(exchanges):
    open_order_requests = []

    for exchange in exchanges:
        open_order_requests.append(exchange.get_open_orders_req())

    return open_order_requests


def format_order(order):
    if order:
        chunk = u"{0:15} | {1:15} | {2:15}".format(order.price, order.volume, order.price * order.volume.amount)
        chunk = exchange_color(chunk, order.exchange.name)
        if order.status == ExchangeOrder.FLAGGED:
            chunk = tc.colored(chunk, None, attrs=['reverse'])
        return chunk
    else:
        return u"{0:51}".format("")


def format_order_book(bids, asks):
    output = u""

    output += u"{0:51}     {1:51}\n".format("           Bids", "           Asks")
    output += u"{0:15} | {1:15} | {2:15} ||| {0:15} | {1:15} | {2:15}\n".format("Price", "Amount", "Value")

    # make copies so we're not popping from the original lists
    bids = list(bids)
    asks = list(asks)
    while bids or asks:
        bid = ask = None
        if bids:
            bid = bids.pop(0)
        output += format_order(bid)
        output += u" ||| "

        if asks:
            ask = asks.pop(0)
        output += format_order(ask)
        output += u"\n"

    return output


def order_book(exchange_name, include_our_orders=False, include_fees=False):
    if exchange_name:
        exchange = make_exchange_from_key(exchange_name)
        exchanges = [exchange]
    else:
        exchanges = all_exchanges()

    obr = start_order_book_requests(exchanges)
    if include_our_orders:
        oor = start_open_order_requests(exchanges)
    
    asks = []
    bids = []
    for exchange in exchanges:
        ob_req = obr.pop(0)
        if include_our_orders:
            oo_req = oor.pop(0)
        try:
            order_book = exchange.get_orderbook_resp(ob_req)
            if include_our_orders:
                open_orders = exchange.open_orders_resp(oo_req)
                order_book = exchange.remove_orders_from_orderbook(order_book, open_orders, only_flag=True)

            asks += order_book['asks'][:20]
            bids += order_book['bids'][:20]
        except (ExchangeAPIErrorException, ExchangeAPIFailureException) as e:
            logger.error("Orderbook fetch failed for %s" % exchange.friendly_name)
    asks.sort(key = lambda x: x.price.to("USD"))
    bids.sort(key = lambda x: x.price.to("USD"), reverse=True)
    
    if include_fees:
        for order in asks + bids:
            order.apply_fee()
    
    print
    # if we are showing a consolidated orderbook
    if not exchange_name:
        print(legend() + "\n")
        for ask in asks:
            ask.price = ask.price.to("USD")
        for bid in bids:
            bid.price = bid.price.to("USD")

    print(format_order_book(bids, asks))



from collections import defaultdict

from sqlalchemy import func

from gryphon.lib.logger import get_logger
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.models.exchange import Exchange, Balance
from gryphon.lib.money import Money

logger = get_logger(__name__)

def get_all_fees_in_period_by_exchange_in_usd(db, start, end):
    fees_by_exchange = db\
        .query(
            func.sum(Trade.fee_in_usd),
            Order._exchange_name,
        )\
        .join(Order)\
        .filter(Trade.time_created >= start)\
        .filter(Trade.time_created < end)\
        .group_by(Order._exchange_name)\
        .all()

    exchange_fees = defaultdict(lambda: Money(0, 'USD'))

    for fees, exchange_name in fees_by_exchange:
        exchange_fees[exchange_name] = Money(fees, 'USD')

    return exchange_fees

def get_all_fees_in_period_in_usd(db, start, end):
    fees = db\
        .query(func.sum(Trade.fee_in_usd))\
        .join(Order)\
        .filter(Trade.time_created >= start)\
        .filter(Trade.time_created < end)\
        .scalar()

    if fees is not None:
        fees = Money(fees, 'USD')
    else:
        fees = Money(0, 'USD')

    return fees

def get_wire_fees_in_period_by_currency(db, start, end):
    wire_fee_results = db\
        .query(Transaction._fee_currency, func.sum(Transaction._fee))\
        .filter(Transaction.time_created >= start)\
        .filter(Transaction.time_created < end)\
        .filter(Transaction._fee != None)\
        .group_by(Transaction._fee_currency)\
        .all()

    wire_fees = Balance()

    for fee_currency, fee_amount in wire_fee_results:
        wire_fees[fee_currency] = Money(fee_amount, fee_currency)

    return wire_fees


def get_all_wire_fees_in_period_in_usd(db, start, end):
    fees_by_currency = get_wire_fees_in_period_by_currency(db, start, end)

    return fees_by_currency.total_usd_value()


def get_wire_fees_in_period_by_exchange(db, start, end):
    wire_fee_results = db\
        .query(
            Transaction._fee_currency,
            func.sum(Transaction._fee),
            Exchange.name,
        )\
        .join(Exchange)\
        .filter(Transaction.time_created >= start)\
        .filter(Transaction.time_created < end)\
        .filter(Transaction._fee != None)\
        .group_by(Exchange.name, Transaction._fee_currency)\
        .all()

    wire_fees = {}

    for fee_currency, fee_amount, exchange_name in wire_fee_results:
        wire_fees[exchange_name] = Money(fee_amount, fee_currency)

    return wire_fees

def get_matched_trading_fees_in_period(db, start_time, end_time):
    """
    This function gives you total trading fees for trades who's p&l was realized in
    the period under question. This means that fees to open the position at the
    beginning of this period are included, and fees to open the position at the end
    of the period are removed.
    """

    fees = get_all_fees_in_period_in_usd(db, start_time, end_time)

    start_position, end_position = get_start_and_end_position(db, start_time, end_time)

    start_open_position_trades, end_open_position_trades = get_start_and_end_position_trades(db, start_time, end_time, start_position, end_position)

    start_position_fee = sum(
        [t.fee_in_usd for t in start_open_position_trades]
    )

    end_position_fee = sum(
        [t.fee_in_usd for t in end_open_position_trades]
    )

    fees = fees + start_position_fee
    fees = fees - end_position_fee

    return fees



from collections import defaultdict
import logging
import tornado.web

from delorean import Delorean

from gryphon.dashboards.handlers.admin_base import AdminBaseHandler
from gryphon.dashboards.handlers.mixins.active_exchanges import ActiveExchangesMixin
from gryphon.dashboards.handlers.mixins.start_and_end_time import StartAndEndTimeMixin
from gryphon.lib.models.emeraldhavoc.trade import Trade as EHTrade
from gryphon.lib.models.trade import Trade

logger = logging.getLogger(__name__)


class TradeViewHandler(AdminBaseHandler, StartAndEndTimeMixin, ActiveExchangesMixin):
    @tornado.web.authenticated
    def get(self):
        start_time, end_time = self.get_start_time_and_end_time()
        active_exchanges = self.get_active_exchanges()
        highlight_ours = self.get_argument('highlight_ours', 'false') == 'true'
        show_sides = self.get_argument('show_sides', 'false') == 'true'

        start_timestamp = Delorean(start_time, 'UTC').epoch * 1000
        end_timestamp = Delorean(end_time, 'UTC').epoch * 1000

        our_trades = self.get_our_trades(start_time, end_time)

        trade_data_by_exchange = self.get_exchange_trades_for_period(
            start_time,
            end_time,
            our_trades,
        )

        self.render_template(
            'trade_view.html',
            args={
                'active_exchanges': active_exchanges,
                'trade_data_by_exchange': trade_data_by_exchange,
                'start_timestamp': start_timestamp,
                'end_timestamp': end_timestamp,
                'highlight_ours': highlight_ours,
                'show_sides': show_sides,
            },
        )

    def get_our_trades(self, start_time, end_time):
        our_trades = self.trading_db.query(
            Trade.exchange_trade_id,
            Trade.trade_type,
        ).filter(Trade.time_created > start_time)\
            .filter(Trade.time_created <= end_time)\
            .filter(Trade.exchange_trade_id != None)\
            .all()

        trade_dict = {}

        for exchange_trade_id, trade_type in our_trades:
            trade_dict[exchange_trade_id] = trade_type

        return trade_dict

    def get_exchange_trades_for_period(self, start_time, end_time, our_trades):
        trades = self.gds_db.query(EHTrade)\
            .filter(EHTrade.timestamp > start_time)\
            .filter(EHTrade.timestamp <= end_time)\
            .all()

        trade_data_by_exchange = defaultdict(lambda: defaultdict(lambda: []))

        for t in trades:
            timestamp = Delorean(t.timestamp, 'UTC').epoch * 1000
            price = float(t.price.to('USD').amount),
            volume = float(t.volume.amount)

            trade_data_by_exchange[t.exchange]['prices'].append([
                timestamp,
                price,
            ])

            trade_data_by_exchange[t.exchange]['volumes'].append(volume)

            if t.exchange_trade_id in our_trades.keys():
                trade_data_by_exchange[t.exchange]['ours'].append(True)

                trade_data_by_exchange[t.exchange]['trade_type'].append(
                    our_trades[t.exchange_trade_id],
                )
            else:
                trade_data_by_exchange[t.exchange]['ours'].append(False)
                trade_data_by_exchange[t.exchange]['trade_type'].append(None)

        return trade_data_by_exchange




# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.volume.bitfinex_volume_poller import BitfinexVolume
from gryphon.data_service.pollers.volume.bitstamp_volume_poller import BitstampVolume
from gryphon.data_service.pollers.volume.coinbase_cad_volume_poller import CoinbaseCADVolume
from gryphon.data_service.pollers.volume.coinbase_volume_poller import CoinbaseVolume
from gryphon.data_service.pollers.volume.itbit_volume_poller import ItbitVolume
from gryphon.data_service.pollers.volume.kraken_cad_volume_poller import KrakenCADVolume
from gryphon.data_service.pollers.volume.kraken_usd_volume_poller import KrakenUSDVolume
from gryphon.data_service.pollers.volume.kraken_volume_poller import KrakenVolume
from gryphon.data_service.pollers.volume.okcoin_volume_poller import OkcoinVolume
from gryphon.data_service.pollers.volume.quadriga_volume_poller import QuadrigaVolume
from gryphon.data_service.task import Task


class VolumePollTask(Task):
    def __init__(self, exchanges=[]):
        exchange_pollers = {
            'KRAKEN': KrakenVolume(),
            'KRAKEN_USD': KrakenUSDVolume(),
            'KRAKEN_CAD': KrakenCADVolume(),
            'ITBIT': ItbitVolume(),
            'QUADRIGA': QuadrigaVolume(),
            'OKCOIN': OkcoinVolume(),
            'BITFINEX': BitfinexVolume(),
            'COINBASE_BTC_USD': CoinbaseVolume(),
            'BITSTAMP_BTC_USD': BitstampVolume(),
            'COINBASE_CAD': CoinbaseCADVolume(),
        }

        if exchanges:
            self.pollers = [
                exchange_pollers[ex]
                for ex in exchanges
                if ex in exchange_pollers
            ]
        else:
            self.pollers = exchange_pollers.values()

    def start_task(self):
        for poller in self.pollers:
            poller.start()



from exponential_moving_average import ExpMovingAverage
def computeMACD(x, slow=26, fast=12):
    """
    compute the MACD (Moving Average Convergence/Divergence) using a fast and slow exponential moving avg'
    return value is emaslow, emafast, macd which are len(x) arrays
    """
    emaslow = ExpMovingAverage(x, slow)
    emafast = ExpMovingAverage(x, fast)
    return emaslow, emafast, emafast - emaslow



"""
A very simple execution strategy meant to demonstrate the implementation of an
execution strategy on the gryphon framework. This strategy does nothing except accept
an exchange name, BID/ASK, and a volume from the command line, and executes the implied
order on the given exchange through a market order.
"""

from gryphon.execution.strategies.base import Strategy


class TrivialExecutionStrategy(ExecutionStrategy):
    def tick(self):
        exchange = self.harness.exchange.from_key(self.args['exchange'])
        exchange.market_order(self.args['mode'], self.args['volume'])




import functools
from delorean import Delorean
from gryphon.lib.logger import get_logger
logger = get_logger(__name__)

from gryphon.lib.session import get_a_memcache_connection

def _localcache_get(localcache, key):
    result = localcache.get(key)
    if not result:
        return None
    value, expiry_time = result
    if Delorean().epoch > expiry_time:
        logger.debug('Localcache for %s expired' % key)
        return None
    return value

def _localcache_set(localcache, key, value, time):
    expiry_time = Delorean().epoch + time
    logger.debug("Setting localcache %s" % expiry_time)
    localcache[key] = [value, expiry_time]

def cache_me(time=1200, ignore_self=False):
    """
    Decorator that caches the result of a method for the specified time in seconds.

    Use it as:

    @cache_me(time=1200) # 20min
    def functionToCache(arguments):
        ...

    """
    def decorator(function):
        localcache = function.localcache = {}
        memcache = get_a_memcache_connection()
        @functools.wraps(function)
        def wrapper(*args, **kwargs):
            if ignore_self:
                key_args = args[1:]
            else:
                key_args = args
            key = '%s%s%s' % (function.__name__, str(key_args), str(kwargs))
            key = key.replace(" ", "_") # Memcache doesn't allow spaces in keys

            # try localcache first
            local_value = _localcache_get(localcache, key)
            logger.debug('Localcache lookup for %s, found? %s', key, local_value != None)
            if local_value:
                return local_value

            # then try memcache
            memcache_value = memcache.get(key)
            logger.debug('Memcache lookup for %s, found? %s', key, memcache_value != None)
            if memcache_value:
                _localcache_set(localcache, key, memcache_value, time)
                return memcache_value

            # otherwise calculate it
            calculated_value = function(*args, **kwargs)
            memcache.set(key, calculated_value, time=time)
            _localcache_set(localcache, key, calculated_value, time)
            return calculated_value
        return wrapper
    return decorator



import pyximport; pyximport.install()

import json
import os
import subprocess

from delorean import epoch
from raven import Client

from gryphon.data_service.consts import *
from gryphon.data_service.queue_consumer import QueueConsumer
from gryphon.lib import session
from gryphon.lib.models.emeraldhavoc.exchange_volume import ExchangeVolume
from gryphon.lib.money import Money


s = Client(dsn=os.environ.get('SENTRY_DSN'))


def exchange_volumes_consumer_function(message, db):
    subprocess.call(['touch', 'monit/heartbeat/exchange_volumes_consumer.txt'])

    exchange_volume_json = json.loads(message)
    timestamp = epoch(exchange_volume_json['timestamp']).datetime
    exchange = exchange_volume_json['exchange_name']
    exch_vol_money = Money(exchange_volume_json['volume'], 'BTC')

    t = ExchangeVolume(
        exchange_volume=exch_vol_money,
        exchange=exchange,
        timestamp=timestamp,
    )

    db.add(t)

    session.commit_mysql_session(db)


def main():
    db = session.get_a_gds_db_mysql_session()

    try:
        volume_consumer = QueueConsumer(
            os.environ.get('AMPQ_ADDRESS'),
            exchange_volumes_consumer_function,
            db,
            EXCHANGE,
            EXCHANGE_TYPE,
            EXCHANGE_VOLUME_BINDING_KEY,
            EXCHANGE_VOLUME_QUEUE,
        )

        volume_consumer.run()
    except KeyboardInterrupt:
        volume_consumer.stop()
    except:
        s.captureException()
    finally:
        db.remove()


if __name__ == '__main__':
    main()



from gryphon.data_service.pollers.orderbook.websocket.coinbase_orderbook_websocket import CoinbaseOrderbookWebsocket


class CoinbaseCADOrderbookWebsocket(CoinbaseOrderbookWebsocket):

    def __init__(self):
        super(CoinbaseCADOrderbookWebsocket, self).__init__()
        self.exchange_name = u'COINBASE_BTC_CAD'
        self.product_id = 'BTC-CAD'



from gryphon.data_service.pollers.orderbook.orderbook_poller import OrderbookPoller


class OkCoinOrderbook(OrderbookPoller):
    def __init__(self):
        self.exchange_name = u'OKCOIN_BTC_USD'
        self.url = 'https://www.okcoin.com/api/v1/depth.do?merge=0&symbol=btc_usd'
        self.poll_time = 2

    # API response format:
    # {"bids":[[787.1, 0.35], ...

    def parse_order(self, order):
        return [order[0], order[1], '']



from twisted.python import log

from gryphon.data_service.pollers.volume.volume_poller import VolumePoller
from gryphon.lib.exchange.kraken_btc_eur import KrakenBTCEURExchange


class KrakenVolume(VolumePoller):
    def __init__(self):
        self.exchange_name = u'KRAKEN_BTC_EUR'
        self.currency = 'EUR'

    @property
    def pair(self):
        return KrakenBTCEURExchange.construct_pair(self.currency)

    @property
    def url(self):
        return 'https://api.kraken.com/0/public/Ticker?pair=%s' % self.pair

    def parse_volume_json(self, volume_dict):
        try:
            return volume_dict['result'][self.pair]['v'][1]
        except Exception as e:
            log.msg('Error getting Kraken Volume %s' % e)



"""
Gryphon Command Line Interface.
===

This is a simple file that is used to create a custom python console with lots of useful
imports and database or exchange connections already available to the user. It's
available as soon as you install gryphon through the command line with:
    gryphon-cli

Note that to use the gryphon console you should have a .env in your current working
directory. That provides the necessary credentials for any connections you want to use.
For this reason we recommend habitually only starting the console from a single
directory, which might be the same directory that you keep your trading .env file and
strategy config files.
"""

import pyximport; pyximport.install()

from datetime import datetime, timedelta
import logging

from cdecimal import *

from gryphon.lib import environment
from gryphon.lib import session
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.logger import get_logger
from gryphon.lib.money import Money
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.models.event import Event
from gryphon.lib.models.exchange import Exchange as ExchangeData
from gryphon.lib.models.liability import Liability
from gryphon.lib.time_parsing import parse

# There's a bizarre bug I haven't had time to investigate whereby if we import
# Delorean in order with the rest of the 3rd party libaries, our time_parsing lib
# doesn't work. This only happens in the console. Bizarre.
from delorean import Delorean

logger = get_logger(__name__)

NO_GDS_ERROR = """\
Couldn't get a gds session. Probably missing credentials or GDS isn't set up.\
"""

# Load the contents of the .env.
environment.load_environment_variables()

# Create oft-used database objects.
trading_db = session.get_a_trading_db_mysql_session()

try:
    gds_db = session.get_a_gds_db_mysql_session()
except Exception as e:
    logger.info(NO_GDS_ERROR)


# Create the oft-used exchange connection shortcuts.
console_shortcuts = {
  'COINBASE_BTC_USD': 'cb',
  'BITSTAMP_BTC_USD': 'b',
  'BITSTAMP_ETH_USD': 'b_eth',
  'GEMINI_BTC_USD': 'g',
  'GEMINI_ETH_USD': 'g_eth',
  'ITBIT_BTC_USD': 'i',
  'KRAKEN_BTC_USD': 'k',
  'KRAKEN_BTC_EUR': 'k_eur',
  'KRAKEN_BTC_CAD': 'k_cad',
}

exchanges = {}

exchange_wrappers = exchange_factory.get_all_initialized_exchange_wrappers(trading_db)

for exchange in exchange_wrappers:
    if exchange.name in console_shortcuts.keys():
        shortcut_name = console_shortcuts[exchange.name]
        globals()[shortcut_name] = exchange
        exchanges[shortcut_name] = exchange


def main():
    from IPython import embed

    embed()


if __name__ == '__main__':
    main()

